import 'package:flutter/material.dart';
import 'package:common_module/utils/xapp_utils.dart';
import 'package:life_module/utils/filter_panel_utils.dart';
import 'package:mvp/mvp.dart';
import 'package:life_module/models/filter_model.dart';

class FilterArea extends StatefulWidget {
  final FilterModel? model;
  final Function(bool checked)? onTap;
  final Function()? onChange;
  final double? topOffset;
  FilterArea({this.model, this.onTap, this.topOffset, this.onChange});
  @override
  State<StatefulWidget> createState() => _FilterAreaState();
}

class _FilterAreaState extends State<FilterArea> {

  Function()? _onChange;

  @override
  void initState() {
    this._onChange = widget.onChange;
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return QMProvider<FilterModel>.value(
      model: widget.model!,
      builderWidget: (context, model, child) {
        if (model.isEmpty()) {
          return Container(
            height: 48.rpx,
          );
        }
        String? regionName = '';
        if (model.getCheckedTwoRegion() != null &&
            model.getCheckedTwoRegion()!.name != null) {
          regionName = model.getCheckedTwoRegion()!.name;
        } else if (model.getCheckedOneRegion() != null) {
          regionName = model.getCheckedOneRegion()!.name;
        }
        return Container(
          height: 48.rpx,
          padding: EdgeInsets.symmetric(horizontal: 1.rpx),
          color: rgba(245, 245, 245, 1),
          transform: Matrix4.translationValues(0, -1, 0),
          child: Row(
            children: [
              _FilterItem(
                text: model.getCheckedOneCategory()!.name,
                checked: model.getFilterAreaIndex() == 1,
                onTap: (checked) {
                  if (widget.onTap != null) {
                    widget.onTap!(checked);
                  }
                  if (checked) {
                    return;
                  }
                  model.stFilterAreaIndex(1);
                  FilterPanelUtils.show(
                      topOffset: widget.topOffset,
                      items: model.getCategorys(),
                      oneCheckedId: model.getCheckedOneCategory()!.id,
                      twoCheckedId: model.getCheckedTwoCategory()!.id,
                      context: context,
                      model: model,
                      onClose: () {
                        model.stFilterAreaIndex(0);
                      },
                      onChange:this._onChange,
                      onConfirm: (onwItem, twoItem) {
                        model.stFilterAreaIndex(0);
                        model.setCheckedCategory(onwItem, twoItem);
                        if(_onChange != null){
                          _onChange!();
                        }
                      });
                },
              ),
              _FilterItem(
                text: regionName,
                checked: model.getFilterAreaIndex() == 2,
                onTap: (checked) {
                  if (widget.onTap != null) {
                    widget.onTap!(checked);
                  }
                  if (checked) {
                    return;
                  }
                  model.stFilterAreaIndex(2);
                  FilterPanelUtils.show(
                      topOffset: widget.topOffset,
                      items: model.getRegions(),
                      oneCheckedId: model.getCheckedOneRegion()!.id,
                      twoCheckedId: model.getCheckedTwoRegion()!.id,
                      context: context,
                      model: model,
                      onClose: () {
                        model.stFilterAreaIndex(0);
                      },
                      onChange:this._onChange,
                      onConfirm: (onwItem, twoItem) {
                        model.stFilterAreaIndex(0);
                        model.setCheckedRegion(onwItem, twoItem);
                        if(_onChange != null){
                          _onChange!();
                        }
                      });
                },
              ),
              _FilterItem(
                text: model.getCheckedOneSort()!.name,
                checked: model.getFilterAreaIndex() == 3,
                onTap: (checked) {
                  if (widget.onTap != null) {
                    widget.onTap!(checked);
                  }
                  if (checked) {
                    return;
                  }
                  model.stFilterAreaIndex(3);
                  FilterPanelUtils.show(
                      topOffset: widget.topOffset,
                      items: model.getSorts(),
                      oneCheckedId: model.getCheckedOneSort()!.id,
                      twoCheckedId: model.getCheckedTwoSort()!.id,
                      context: context,
                      model: model,
                      onClose: () {
                        model.stFilterAreaIndex(0);
                      },
                      onChange:this._onChange,
                      onConfirm: (onwItem, twoItem) {
                        model.stFilterAreaIndex(0);
                        model.setCheckedSort(onwItem, twoItem);
                        if(_onChange != null){
                          _onChange!();
                        }
                      });
                },
              ),
            ],
          ),
        );
      },
    );
  }
}

class _FilterItem extends StatelessWidget {
  final bool checked;
  final String? text;
  final Function(bool checked)? onTap;
  _FilterItem({this.text, this.onTap, this.checked = false});
  @override
  Widget build(BuildContext context) {
    Color textColor = rgba(48, 38, 0, 1);
    Color iconColor = rgba(151, 151, 151, 1);
    if (checked) {
      textColor = rgba(255, 128, 0, 1);
      iconColor = textColor;
    }
    String clipText = text!;
    if (clipText.length > 5) {
      clipText = "${text!.substring(0, 5)}...";
    }
    return Expanded(
      child: GestureDetector(
        onTap: () {
          onTap!(checked);
        },
        child: Container(
          height: 48.rpx,
          padding: EdgeInsets.only(
              left: 6.rpx,
              right: 6.rpx,
              bottom: checked ? 0 : 8.rpx,
              top: 12.rpx),
          child: Container(
            padding: EdgeInsets.only(
              bottom: checked ? 8.rpx : 0,
            ),
            decoration: BoxDecoration(
                color: Colors.white,
                borderRadius: BorderRadius.only(
                    topLeft: Radius.circular(4.rpx),
                    topRight: Radius.circular(4.rpx),
                    bottomLeft: Radius.circular(checked ? 0 : 4.rpx),
                    bottomRight: Radius.circular(checked ? 0 : 4.rpx))),
            child: Row(
              mainAxisAlignment: MainAxisAlignment.center,
              children: [
                Container(
                  child: Text(
                    clipText,
                    overflow: TextOverflow.clip,
                    style: TextStyle(
                        color: textColor, fontSize: 14.rpx, height: 1.rpx),
                  ),
                ),
                Icon(checked ? Icons.arrow_drop_up : Icons.arrow_drop_down,
                    size: 13.rpx, color: iconColor)
              ],
            ),
          ),
        ),
      ),
    );
  }
}
