import 'package:flutter/material.dart';
import 'package:common_module/utils/xapp_utils.dart';
import 'package:life_module/components/filter_area.dart';
import 'package:life_repository/life_repository.dart';
import 'package:life_module/models/filter_model.dart';

class FilterPanel extends StatelessWidget {
  final FilterModel? model;
  final List<FilterItemEntity>? items;
  final String? oneCheckedId;
  final String? twoCheckedId;
  final Function()? onClose;
  final double? topOffset;
  final Function()? onChange;
  final Function(FilterItemEntity? onwItem, FilterItemEntity? twoItem) onConfirm;
  FilterPanel(
      {this.items,
      this.model,
      this.oneCheckedId,
      this.twoCheckedId,
      this.onClose,
      this.topOffset,
      required this.onConfirm,
      this.onChange,
    });
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.transparent,
      body: SafeArea(
        bottom: false,
        child: Column(
          children: [
            GestureDetector(
              onTap: onClose,
              child: Container(
                color: Colors.transparent,
                height: 44.rpx,
              ),
            ),
            FilterArea(
              model: model,
              onTap: (checked) {
                onClose!();
              },
              onChange:onChange,
            ),
            Expanded(
              child: Container(
                color: rgba(0, 0, 0, 0.4),
                child: Column(
                  children: [
                    _Body(
                      list: items,
                      oneCheckedId: oneCheckedId,
                      twoCheckedId: twoCheckedId,
                      onConfirm: onConfirm,
                    ),
                    Expanded(
                      child: GestureDetector(
                          onTap: onClose,
                          child: Container(
                            color: Colors.transparent,
                          )),
                    )
                  ],
                ),
              ),
            ),
          ],
        ),
      ),
    );
  }
}

class _Body extends StatefulWidget {
  final String? oneCheckedId;
  final String? twoCheckedId;
  final List<FilterItemEntity>? list;
  final Function(FilterItemEntity? onwItem, FilterItemEntity? twoItem) onConfirm;
  _Body({this.list, this.oneCheckedId, this.twoCheckedId,required this.onConfirm});
  @override
  State<StatefulWidget> createState() => _BodyState();
}

class _BodyState extends State<_Body> {
  String? oneCheckedId = '';
  String? twoCheckedId = '';
  FilterItemEntity? onwItem;
  FilterItemEntity? twoItem;
  List<FilterItemEntity>? get list => widget.list;
  List<FilterItemEntity>? get twoList {
    FilterItemEntity item = list!.firstWhere((item) => item.id == oneCheckedId);
    if (item != null && item.subs!.length > 0) {
      return item.subs;
    }
    return [];
  }

  // 是否需要二级
  bool get isTwoLevel {
    for (int i = 0, len = list!.length; i < len; i++) {
      FilterItemEntity item = list![i];
      if (item.subs != null && item.subs!.length > 0) {
        return true;
      }
    }
    return false;
  }

  @override
  void initState() {
    super.initState();
    oneCheckedId = widget.oneCheckedId;
    twoCheckedId = widget.twoCheckedId;
    onwItem = list!.firstWhere((item) => item.id == oneCheckedId);
    if (twoList != null && twoList!.length > 0) {
      twoItem = twoList!.firstWhere((item) => item.id == twoCheckedId);
    }
  }

  @override
  Widget build(BuildContext context) {
    return Expanded(
      child: Container(
        child: Container(
          height: 342.rpx,
          transform: Matrix4.translationValues(0, -1.rpx, 0),
          decoration: BoxDecoration(
              color: Colors.white,
              borderRadius:
                  BorderRadius.vertical(bottom: Radius.circular(12.rpx))),
          child: Container(
            child: Column(
              children: [
                Builder(
                  builder: (context) {
                    if (!isTwoLevel) {
                      return _SortLevel(
                        list: list,
                        checkedId: oneCheckedId,
                        onTap: (item) {
                          if (oneCheckedId == item.id) {
                            return;
                          }
                          onwItem = item;
                          setState(() {
                            oneCheckedId = item.id;
                            twoCheckedId = '';
                          });
                        },
                      );
                    }
                    return Expanded(
                        child: Row(
                      children: [
                        _OneLevel(
                          list: list,
                          checkedId: oneCheckedId,
                          onTap: (item) {
                            if (oneCheckedId == item.id) {
                              return;
                            }
                            onwItem = item;
                            if (onwItem!.subs != null &&
                                onwItem!.subs!.length > 0) {
                              twoItem = onwItem!.subs![0];
                            } else {
                              twoItem = FilterItemEntity(null, null, []);
                            }
                            setState(() {
                              oneCheckedId = item.id;
                              twoCheckedId = twoItem!.id;
                            });
                          },
                        ),
                        _TwoLevel(
                          list: twoList,
                          checkedId: twoCheckedId,
                          onTap: (item) {
                            if (twoCheckedId == item.id) {
                              return;
                            }
                            twoItem = item;
                            setState(() {
                              twoCheckedId = item.id;
                            });
                          },
                        )
                      ],
                    ));
                  },
                ),
                _Fotter(
                  onReset: () {
                    FilterItemEntity onwItem = list![0];
                    FilterItemEntity twoItem;
                    if (onwItem.subs != null && onwItem.subs!.length > 0) {
                      twoItem = onwItem.subs![0];
                    } else {
                      twoItem = FilterItemEntity(null, null, []);
                    }
                    widget.onConfirm!(onwItem, twoItem);
                  },
                  onConfirm: () {
                    widget.onConfirm!(onwItem, twoItem);
                  },
                ),
              ],
            ),
          ),
        ),
      ),
    );
  }
}

/// 一级选择列表
class _OneLevel extends StatelessWidget {
  final String? checkedId;
  final List<FilterItemEntity>? list;
  final Function(FilterItemEntity item)? onTap;
  _OneLevel({this.list, this.checkedId, this.onTap});
  @override
  Widget build(BuildContext context) {
    int len = list!.length;
    return Container(
      width: 94.rpx,
      child: ListView.builder(
          itemCount: len,
          physics: ClampingScrollPhysics(),
          itemBuilder: (c, i) {
            // 上一个索引
            int preIndex = i - 1;
            // 下一个索引
            int nextIndex = i + 1;
            FilterItemEntity item = list![i];
            bool checked = item.id == checkedId;
            bool preChecked = preIndex > -1 && list![preIndex].id == checkedId;
            bool nextChecked =
                nextIndex < len - 1 && list![nextIndex].id == checkedId;
            return _OneLevelTextItem(
              text: item.name,
              checked: checked,
              nextChecked: nextChecked,
              preChecked: preChecked,
              onTap: () {
                onTap!(item);
              },
            );
          }),
    );
  }
}

/// 二级选择列表
class _TwoLevel extends StatelessWidget {
  final String? checkedId;
  final List<FilterItemEntity>? list;
  final Function(FilterItemEntity item)? onTap;
  _TwoLevel({this.list, this.checkedId, this.onTap});
  @override
  Widget build(BuildContext context) {
    if (list == null || list!.length == 0) {
      return Container();
    }
    return Expanded(
      child: GridView.builder(
        physics: ClampingScrollPhysics(),
        gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
            crossAxisCount: 2, childAspectRatio: 3.1818181818181817),
        itemBuilder: (context, i) {
          FilterItemEntity item = list![i];
          bool checked = item.id == checkedId;
          return _TextItem(
            text: item.name,
            checked: checked,
            onTap: () {
              onTap!(item);
            },
          );
        },
        itemCount: list!.length,
      ),
    );
  }
}

/// 排序选择列表
class _SortLevel extends StatelessWidget {
  final String? checkedId;
  final List<FilterItemEntity>? list;
  final Function(FilterItemEntity item)? onTap;
  _SortLevel({this.list, this.checkedId, this.onTap});
  @override
  Widget build(BuildContext context) {
    List<Widget> childs = [];

    List<Widget> rows = [];
    for (int i = 0, len = list!.length; i < len; i++) {
      FilterItemEntity item = list![i];
      bool checked = item.id == checkedId;
      Widget widget = _TextItem(
        text: item.name,
        checked: checked,
        onTap: () {
          onTap!(item);
        },
      );
      rows.add(Container(
        width: 95.rpx,
        child: widget,
      ));

      if (rows.length == 2 || i == len - 1) {
        childs.add(Row(
          children: rows,
        ));
        rows = [];
      }
    }
    return Expanded(
      child: SingleChildScrollView(
        child: Column(
          children: childs,
        ),
      ),
    );
  }
}

/// 一级选择文字
class _OneLevelTextItem extends StatelessWidget {
  final String? text;
  final bool checked;
  final bool preChecked;
  final bool nextChecked;
  final Function()? onTap;
  _OneLevelTextItem(
      {this.text,
      this.onTap,
      this.checked = false,
      this.preChecked = false,
      this.nextChecked = false});
  @override
  Widget build(BuildContext context) {
    Radius radius = Radius.circular(8.rpx);
    Color color = rgba(245, 245, 245, 1);
    if (checked) {
      color = Colors.transparent;
    }
    return Container(
      decoration: BoxDecoration(
          color: color,
          borderRadius: BorderRadius.only(
            topRight: preChecked ? radius : Radius.zero,
            bottomRight: nextChecked ? radius : Radius.zero,
          )),
      child: _TextItem(
        text: text,
        checked: checked,
        onTap: onTap,
      ),
    );
  }
}

/// 二级选择文字
class _TextItem extends StatelessWidget {
  final String? text;
  final bool checked;
  final Function()? onTap;
  _TextItem({this.text, this.onTap, this.checked = false});
  @override
  Widget build(BuildContext context) {
    Color color = rgba(102, 102, 102, 1);
    if (checked) {
      color = rgba(255, 128, 0, 1);
    }
    return GestureDetector(
        onTap: onTap,
        behavior: HitTestBehavior.translucent,
        child: Container(
          height: 44.rpx,
          alignment: Alignment.center,
          child: Text(
            text!,
            style: TextStyle(color: color, fontSize: 13.rpx),
          ),
        ));
  }
}

/// 底部操作按钮
class _Fotter extends StatelessWidget {
  final Function()? onReset;
  final Function()? onConfirm;
  _Fotter({this.onReset, this.onConfirm});
  @override
  Widget build(BuildContext context) {
    return Container(
      height: 56.rpx,
      padding: EdgeInsets.symmetric(horizontal: 13.rpx),
      child: Row(
        children: [
          GestureDetector(
              onTap: onReset,
              child: Container(
                width: 82.rpx,
                height: 40.rpx,
                alignment: Alignment.center,
                decoration: BoxDecoration(
                    color: rgba(245, 245, 245, 1),
                    borderRadius: BorderRadius.circular(8.rpx)),
                child: Text(
                  "重置",
                  style: TextStyle(
                      color: rgba(51, 51, 51, 1),
                      fontSize: 15.rpx,
                      fontWeight: FontWeight.bold,
                      height: 1.rpx),
                ),
              )),
          SizedBox(
            width: 12.rpx,
          ),
          Expanded(
            child: GestureDetector(
              onTap: onConfirm,
              child: Container(
                height: 40.rpx,
                alignment: Alignment.center,
                decoration: BoxDecoration(
                    gradient: LinearGradient(
                        begin: Alignment.topCenter,
                        end: Alignment.bottomCenter,
                        colors: [rgba(255, 187, 0, 1), rgba(255, 187, 0, 1)]),
                    borderRadius: BorderRadius.circular(8.rpx)),
                child: Text(
                  "确认",
                  style: TextStyle(
                      color: rgba(51, 51, 51, 1),
                      fontSize: 15.rpx,
                      fontWeight: FontWeight.bold,
                      height: 1.rpx),
                ),
              ),
            ),
          )
        ],
      ),
    );
  }
}
