import 'package:flutter/material.dart';
import 'package:common_module/utils/xapp_utils.dart';
import 'package:life_module/components/type_icon.dart';
import 'package:life_repository/life_repository.dart';
import 'package:common_module/widget/xiaoxiong_base_image_widget/widget.dart';

class GoodsItem extends StatelessWidget {
  final ComboItem? item;
  final Function()? onTap;
  GoodsItem({this.item, this.onTap});
  @override
  Widget build(BuildContext context) {
    return InkWell(
      onTap: onTap,
      child: Container(
        height: 98.rpx,
        margin: EdgeInsets.only(left: 13.rpx, right: 13.rpx, bottom: 8.rpx),
        decoration: BoxDecoration(
            color: Colors.white, borderRadius: BorderRadius.circular(8.rpx)),
        child: Row(
          children: [
            SizedBox(
              width: 12.rpx,
            ),
            ClipRRect(
                borderRadius: BorderRadius.circular(8.rpx),
                child: Container(
                  height: 74.rpx,
                  width: 74.rpx,
                  child: XiaoxiongBaseImageWidget(
                    imageUrl: item!.shopImg!,
                    height: 74.rpx,
                    width: 74.rpx,
                  ),
                )),
            SizedBox(
              width: 10.rpx,
            ),
            Expanded(
              child: Container(
                height: 74.rpx,
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    _Title(item),
                    _PriceInfo(item),
                    _LocaionInfo(item)
                  ],
                ),
              ),
            ),
            SizedBox(
              width: 12.rpx,
            ),
          ],
        ),
      ),
    );
  }
}

//、 标题
class _Title extends StatelessWidget {
  final ComboItem? item;
  _Title(this.item);
  @override
  Widget build(BuildContext context) {
    return Container(
      height: 22.rpx,
      child: Row(
        children: [
          TypeIcon(
            type: int.parse(item!.dealType!),
          ),
          SizedBox(
            width: 5.rpx,
          ),
          Expanded(
              child: Container(
            child: Text(
              item!.itemTitle!,
              overflow: TextOverflow.ellipsis,
              maxLines: 1,
              style: TextStyle(
                  fontSize: 15.rpx, fontWeight: FontWeight.bold, height: 1.rpx),
            ),
          ))
        ],
      ),
    );
  }
}

/// 价格信息
class _PriceInfo extends StatelessWidget {
  final ComboItem? item;
  _PriceInfo(this.item);
  @override
  Widget build(BuildContext context) {
    return Container(
      margin: EdgeInsets.only(top: 6.rpx),
      height: 19.rpx,
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.baseline,
        children: [
          Text(
            "¥",
            style: TextStyle(
              color: rgba(255, 4, 0, 1),
              fontSize: 11.rpx,
            ),
          ),
          Text(
            item!.endPrice!,
            style: TextStyle(
              color: rgba(255, 4, 0, 1),
              fontSize: 16.rpx,
            ),
          ),
          SizedBox(
            width: 4.rpx,
          ),
          Text(
            "¥${item!.originalPrice}",
            style: TextStyle(
                color: rgba(153, 153, 153, 1),
                fontSize: 12.rpx,
                decoration: TextDecoration.lineThrough),
          ),
          SizedBox(
            width: 12.rpx,
          ),
          Container(
            padding: EdgeInsets.symmetric(horizontal: 4.rpx, vertical: 2.rpx),
            decoration: BoxDecoration(
                border: Border.all(color: rgba(255, 128, 0, 1), width: 0.5),
                borderRadius: BorderRadius.only(
                    topLeft: Radius.circular(9.rpx),
                    topRight: Radius.circular(9.rpx),
                    bottomRight: Radius.circular(9.rpx))),
            child: Text(
              item!.userCommissionText!,
              style: TextStyle(
                  color: rgba(255, 128, 0, 1), fontSize: 11.rpx, height: 1.2),
            ),
          ),
        ],
      ),
    );
  }
}

/// 定位信息
class _LocaionInfo extends StatelessWidget {
  final ComboItem? item;
  _LocaionInfo(this.item);
  @override
  Widget build(BuildContext context) {
    return Container(
      margin: EdgeInsets.only(top: 9.rpx),
      height: 18.rpx,
      width: double.infinity,
      child: Row(
        children: [
          Container(
            constraints: BoxConstraints(maxWidth: 190.rpx),
            child: Text(
              item!.shopTitle!,
              maxLines: 1,
              overflow: TextOverflow.ellipsis,
              style: TextStyle(
                  color: rgba(102, 102, 102, 1), fontSize: 11.rpx, height: 1.2),
            ),
          ),
          SizedBox(
            width: 2.rpx,
          ),
          /*LifeImage(
            name: "assets/images/life/ic_miaosha_arrow_right.png",
            width: 8.rpx,
            height: 8.rpx,
          ),*/
          Spacer(),
          Text(
            item!.distance!,
            style: TextStyle(
                color: rgba(102, 102, 102, 1), fontSize: 11.rpx, height: 1.2),
          ),
        ],
      ),
    );
  }
}
