import 'package:flutter/material.dart';
import 'package:common_module/utils/xapp_utils.dart';

class SearchBar extends StatefulWidget {
  final Color? inputColor;
  final bool enabled;
  final FocusNode? focusNode;
  final Function(String)? onSubmitted;
  final Function(String)? onChanged;
  final TextEditingController? controller;
  SearchBar(
      {this.enabled = false,
      this.inputColor,
      this.focusNode,
      this.onChanged,
      this.onSubmitted,
      this.controller});
  @override
  State<StatefulWidget> createState() => _SearchBarState();
}

class _SearchBarState extends State<SearchBar> {
  @override
  Widget build(BuildContext context) {
    return Container(
      height: 34.rpx,
      alignment: Alignment.centerLeft,
      decoration: BoxDecoration(
          color: widget.inputColor,
          borderRadius: BorderRadius.circular(17.rpx),
          border: Border.all(color: rgba(255, 128, 0, 1), width: 1)),
      child: Row(
        children: [
          _TextField(
              enabled: widget.enabled,
              onChanged: widget.onChanged,
              onSubmitted: widget.onSubmitted,
              focusNode: widget.focusNode,
              controller: widget.controller),
          !widget.enabled
              ? _Button()
              : InkWell(
                  child: _Button(),
                  onTap: () {
                    widget.onSubmitted!(widget.controller!.text);
                  },
                ),
          SizedBox(
            width: 3.rpx,
          )
        ],
      ),
    );
  }
}

class _TextField extends StatefulWidget {
  final bool? enabled;
  final FocusNode? focusNode;
  final Function(String)? onSubmitted;
  final Function(String)? onChanged;
  final TextEditingController? controller;
  _TextField(
      {this.enabled,
      this.focusNode,
      this.onChanged,
      this.onSubmitted,
      this.controller});

  @override
  State<StatefulWidget> createState() => _TextFieldState();
}

/// 输入框
class _TextFieldState extends State<_TextField> {
  bool showCloseBtn = false;
  @override
  void initState() {
    super.initState();
    if (widget.controller != null) {
      widget.controller!.addListener(() {
        setState(() {
          showCloseBtn = widget.controller!.text.isNotEmpty;
        });
      });
    }
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Expanded(
        child: Container(
      height: 34.rpx,
      alignment: Alignment.centerLeft,
      child: Stack(
        children: [
          Container(
            height: 34.rpx,
            padding: EdgeInsets.only(right: 29.rpx),
            child: TextField(
              enabled: widget.enabled,
              focusNode: widget.focusNode,
              controller: widget.controller,
              onSubmitted: widget.onSubmitted,
              onChanged: widget.onChanged,
              textAlignVertical: TextAlignVertical.center,
              textInputAction: TextInputAction.search,
              keyboardType: TextInputType.text,
              style: TextStyle(color: rgba(48, 38, 0, 1), fontSize: 12.rpx),
              decoration: InputDecoration(
                fillColor: Colors.transparent,
                prefixIconConstraints: BoxConstraints(maxWidth: 26.rpx),
                prefixIcon: Container(
                  margin:
                      EdgeInsets.only(left: 8.rpx, right: 4.rpx, bottom: 2.rpx),
                  child: Image.asset("assets/ic_home_search.png",
                      width: 20.rpx, height: 20.rpx, fit: BoxFit.contain),
                ),
                hintText: '搜索店铺，领隐藏优惠券省钱',
                isDense: false,
                hintStyle:
                    TextStyle(color: rgba(170, 170, 170, 1), fontSize: 12.rpx),
                border: OutlineInputBorder(
                  borderSide: BorderSide(style: BorderStyle.none),
                  gapPadding: 0.0,
                ),
                focusedBorder: OutlineInputBorder(
                  borderSide: BorderSide(style: BorderStyle.none),
                  gapPadding: 0.0,
                ),
                focusedErrorBorder: OutlineInputBorder(
                  borderSide: BorderSide(style: BorderStyle.none),
                  gapPadding: 0.0,
                ),
                enabledBorder: OutlineInputBorder(
                  borderSide: BorderSide(style: BorderStyle.none),
                  gapPadding: 0.0,
                ),
                disabledBorder: OutlineInputBorder(
                  borderSide: BorderSide(style: BorderStyle.none),
                  gapPadding: 0.0,
                ),
                errorBorder: OutlineInputBorder(
                  borderSide: BorderSide(style: BorderStyle.none),
                  gapPadding: 0.0,
                ),
                contentPadding: EdgeInsets.all(0),
              ),
            ),
          ),
          Builder(
            builder: (context) {
              if (!showCloseBtn) {
                return SizedBox();
              }
              return Positioned(
                right: 0,
                top: 0,
                child: InkWell(
                  onTap: () {
                    widget.onChanged!("");
                    widget.onSubmitted!("");
                    widget.controller!.text = '';
                    widget.focusNode!.requestFocus();
                  },
                  child: Container(
                    width: 34.rpx,
                    height: 34.rpx,
                    alignment: Alignment.center,
                    child: Image.asset(
                      "assets/close_btn_black.png",
                      width: 14.rpx,
                      height: 14.rpx,
                    ),
                  ),
                ),
              );
            },
          )
        ],
      ),
    ));
  }
}

/// 搜索按钮
class _Button extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return Container(
      width: 46.rpx,
      height: 28.rpx,
      alignment: Alignment.center,
      decoration: BoxDecoration(
          color: rgba(255, 128, 0, 1),
          borderRadius: BorderRadius.circular(17.rpx)),
      child: Text(
        '搜索',
        style: TextStyle(color: rgba(255, 255, 255, 1), fontSize: 13.rpx),
      ),
    );
  }
}
