import 'package:flutter/material.dart';
import 'package:common_module/utils/xapp_utils.dart';
import 'package:life_module/components/type_icon.dart';
import 'package:life_repository/life_repository.dart';
import 'package:common_module/widget/xiaoxiong_base_image_widget/widget.dart';

class ShopItem extends StatelessWidget {
  final RestaurantItem? item;
  final Function()? onTap;
  ShopItem({this.item, this.onTap});
  @override
  Widget build(BuildContext context) {
    return InkWell(
      onTap: onTap,
      child: Container(
        margin: EdgeInsets.only(left: 13.rpx, right: 13.rpx, bottom: 8.rpx),
        padding: EdgeInsets.symmetric(vertical: 12.rpx),
        alignment: Alignment.topLeft,
        decoration: BoxDecoration(
            color: Colors.white, borderRadius: BorderRadius.circular(8.rpx)),
        child: Row(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            SizedBox(
              width: 12.rpx,
            ),
            ClipRRect(
                borderRadius: BorderRadius.circular(8.rpx),
                child: Container(
                  height: 74.rpx,
                  width: 74.rpx,
                  child: XiaoxiongBaseImageWidget(
                    imageUrl: item!.shopImg!,
                    height: 74.rpx,
                    width: 74.rpx,
                  ),
                )),
            SizedBox(
              width: 10.rpx,
            ),
            Expanded(
              child: Container(
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    _Title(item),
                    _ScoreInfo(item),
                    _LocaionInfo(item),
                    Container(
                      height: 0.5.rpx,
                      color: rgba(151, 151, 151, 0.1),
                    ),
                    _GoodList(item)
                  ],
                ),
              ),
            ),
            SizedBox(
              width: 12.rpx,
            ),
          ],
        ),
      ),
    );
  }
}

/// 标题
class _Title extends StatelessWidget {
  final RestaurantItem? item;
  _Title(this.item);
  @override
  Widget build(BuildContext context) {
    return Container(
      height: 22.rpx,
      child: Row(
        children: [
          Expanded(
              child: Container(
            child: Text(
              item!.shopTitle!,
              overflow: TextOverflow.ellipsis,
              maxLines: 1,
              style: TextStyle(
                  fontSize: 15.rpx, fontWeight: FontWeight.bold, height: 1.rpx),
            ),
          ))
        ],
      ),
    );
  }
}

/// 评分和销量信息
class _ScoreInfo extends StatelessWidget {
  final RestaurantItem? item;
  _ScoreInfo(this.item);
  @override
  Widget build(BuildContext context) {
    var showStarScore = item!.showStarScore!;
    List<InlineSpan> widgets = [];
    double score = double.parse(showStarScore);
    for (int star = 1, len = score.ceil(); star <= len; star++) {
      widgets.add(WidgetSpan(
          child: Image.asset(
        star <= score
            ? "assets/ic_star_small_full.png"
            : "assets/ic_star_small_half.png",
        width: 12.rpx,
        height: 16.rpx,
      )));
    }
    for (int star = widgets.length; star < 5; star++) {
      widgets.add(WidgetSpan(
          child: Image.asset(
        "assets/ic_star_small_blank.png",
        width: 12.rpx,
        height: 16.rpx,
      )));
    }
    widgets.add(WidgetSpan(
        child: SizedBox(
      width: 2.rpx,
    )));
    widgets.add(TextSpan(
        text: double.parse(item!.commentScore!).toStringAsFixed(1),
        style: TextStyle(
            color: rgba(255, 128, 0, 1),
            fontSize: 14.rpx,
            fontWeight: FontWeight.bold)));
    return Container(
      height: 18.rpx,
      margin: EdgeInsets.only(top: 4.rpx),
      child: Row(
        children: [
          Container(
            alignment: Alignment.centerLeft,
            height: 14.rpx,
            child: Text.rich(TextSpan(children: widgets)),
          ),
          SizedBox(
            width: 8.rpx,
          ),
          Text(
            " ",
            style: TextStyle(color: rgba(102, 102, 102, 1), fontSize: 12.rpx),
          ),
          Spacer(),
          Text(
            "¥${item!.avgPrice}/人",
            style: TextStyle(color: rgba(102, 102, 102, 1), fontSize: 11.rpx),
          )
        ],
      ),
    );
  }
}

/// 地址信息
class _LocaionInfo extends StatelessWidget {
  final RestaurantItem? item;
  _LocaionInfo(this.item);
  @override
  Widget build(BuildContext context) {
    return Container(
      height: 18.rpx,
      margin: EdgeInsets.only(top: 7.rpx, bottom: 8.rpx),
      child: Row(
        children: [
          Text(
            item!.categoryName!,
            style: TextStyle(color: rgba(102, 102, 102, 1), fontSize: 11.rpx),
          ),
          Spacer(),
          Text(
            "${item!.regionName}｜${item!.distance}",
            style: TextStyle(color: rgba(102, 102, 102, 1), fontSize: 11.rpx),
          )
        ],
      ),
    );
  }
}

class _GoodList extends StatelessWidget {
  final RestaurantItem? item;
  _GoodList(this.item);
  @override
  Widget build(BuildContext context) {
    List<Widget> items = item!.itemList!.map((e) {
      return _GoodItem(e);
    }).toList();
    return Container(
      child: Column(
        children: items,
      ),
    );
  }
}

/// 代金券和团购单个信息
class _GoodItem extends StatelessWidget {
  final ShopSubItemList item;
  _GoodItem(this.item);
  @override
  Widget build(BuildContext context) {
    return Container(
      height: 16.rpx,
      margin: EdgeInsets.only(top: 9.rpx),
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          TypeIcon(type: int.parse(item.dealType!)),
          SizedBox(
            width: 6.rpx,
          ),
          _PriceInfo(item),
          SizedBox(
            width: 8.rpx,
          ),
          Expanded(
            child: Container(
              child: Text(
                item.itemTitle!,
                overflow: TextOverflow.ellipsis,
                maxLines: 1,
                style: TextStyle(color: rgba(51, 51, 51, 1), fontSize: 12.rpx),
              ),
            ),
          )
        ],
      ),
    );
  }
}

/// 价格信息
class _PriceInfo extends StatelessWidget {
  final ShopSubItemList item;
  _PriceInfo(this.item);
  @override
  Widget build(BuildContext context) {
    return Container(
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.baseline,
        children: [
          Text(
            "¥",
            style: TextStyle(
              color: rgba(255, 4, 0, 1),
              fontSize: 11.rpx,
            ),
          ),
          Text(
            item.endPrice!,
            style: TextStyle(
              color: rgba(255, 4, 0, 1),
              fontSize: 14.rpx,
            ),
          ),
          SizedBox(
            width: 8.rpx,
          ),
          Text(
            "¥${item.originalPrice}",
            style: TextStyle(
                color: rgba(153, 153, 153, 1),
                fontSize: 12.rpx,
                decoration: TextDecoration.lineThrough),
          ),
        ],
      ),
    );
  }
}
