import 'package:flutter/material.dart';
import './shop_item.dart';
import './goods_item.dart';
import '../models/shop_list_model.dart';
import 'package:mvp/mvp.dart';
import 'package:life_repository/life_repository.dart';
import 'package:common_module/utils/list_model_status_utils.dart';
import 'package:common_module/utils/xapp_utils.dart';

class ShopList extends StatelessWidget {
  final ShopListModel? model;
  final Function(ShopItemEntity)? onTap;
  final Function()? onReset;
  ShopList({this.model, this.onTap, this.onReset});
  @override
  Widget build(BuildContext context) {
    return SliverPadding(
      padding: EdgeInsets.zero,
      sliver: QMProvider<ShopListModel>.value(
        model: model!,
        builderWidget: (context, model, child) {
          Widget? statusWidget = ListModelStatusUtils.getInstance()!.form(
              failText: "没有搜到结果~",
              btnText: "恢复默认",
              paddingTop: 30.rpx,
              iconSize: 200.rpx,
              btnRadius: 40.rpx,
              btnColor: Colors.black,
              icon: "assets/life_img_no_location.png",
              model: model,
              onTap: onReset);
          if (statusWidget != null) {
            return SliverFillViewport(
              delegate: SliverChildBuilderDelegate((context, index) {
                return Container(
                  alignment: Alignment.topCenter,
                  child: statusWidget,
                );
              }, childCount: 1),
              viewportFraction: 1.0,
            );
          }
          return SliverList(
              delegate: SliverChildBuilderDelegate(
            (BuildContext context, int index) {
              ShopItemEntity item = model.getList()[index];
              if (item.type == 0) {
                return ShopItem(
                  item: item.restaurant,
                  onTap: () {
                    onTap!(item);
                  },
                );
              }
              return GoodsItem(
                item: item.combo,
                onTap: () {
                  onTap!(item);
                },
              );
            },
            childCount: model.getList().length,
          ));
        },
      ),
    );
  }
}
