import 'dart:async';

import 'package:flutter/material.dart';

import 'package:mvp/mvp.dart';
import 'package:life_repository/life_repository.dart';

class FilterModel extends BaseModel {
  List<FilterItemEntity> _regions = [];
  List<FilterItemEntity> _categorys = [];
  List<FilterItemEntity> _sorts = [];

  FilterItemEntity? _checkedOneRegion;
  FilterItemEntity? _checkedOneCategory;
  FilterItemEntity? _checkedOneSort;

  FilterItemEntity? _checkedTwoRegion;
  FilterItemEntity? _checkedTwoCategory;
  FilterItemEntity? _checkedTwoSort;

  // 筛选区域选中的索引
  int _filterAreaIndex = 0;
  // 二级类目控制器
  TabController? _shopClassNavTabController;
  int? _shopTypeNavIndex = 0;
  // 搜索关键词
  String? _keywords;

  Future<void> getFilterList(String cid) async {
    FilterObject object = await (FilterArepository.get().getFilterList(cid) as FutureOr<FilterObject>);
    _regions = object.regions;
    _categorys = object.categorys;
    _sorts = object.sorts;
    if (_regions.length > 0) {
      _checkedOneRegion = _regions[0];
      if (_checkedOneRegion!.subs!.length > 0) {
        _checkedTwoRegion = _checkedOneRegion!.subs![0];
      }
    }
    if (_categorys.length > 0) {
      _checkedOneCategory = _categorys[0];
      if (_checkedOneCategory!.subs!.length > 0) {
        _checkedTwoCategory = _checkedOneCategory!.subs![0];
        _shopClassNavTabController = TabController(
            length: _checkedOneCategory!.subs!.length, vsync: ScrollableState());
      }
    }
    if (_sorts.length > 0) {
      _checkedOneSort = _sorts[0];
      if (_checkedOneSort!.subs!.length > 0) {
        _checkedTwoSort = _checkedOneSort!.subs![0];
      }
    }

    notifyListeners();
  }

  bool isEmpty() {
    return _regions.length == 0 || _categorys.length == 0 || _sorts.length == 0;
  }

  int getFilterAreaIndex() {
    return _filterAreaIndex;
  }

  void stFilterAreaIndex(int index) {
    _filterAreaIndex = index;
    notifyListeners();
  }

  FilterItemEntity? getCheckedOneRegion() {
    return _checkedOneRegion;
  }

  void setCheckedRegion(FilterItemEntity r1, FilterItemEntity r2) {
    _checkedOneRegion = r1;
    _checkedTwoRegion = r2;
    notifyListeners();
  }

  FilterItemEntity? getCheckedOneCategory() {
    return _checkedOneCategory;
  }

  void setCheckedCategory(FilterItemEntity? c1, FilterItemEntity c2) {
    var oldCheckedOneCategory = _checkedOneCategory;
    _checkedOneCategory = c1;
    _checkedTwoCategory = c2;
    int index = 0;
    if (c2 != null && c1!.subs!.length > 0) {
      index = c1.subs!.indexWhere((item) => item.id == c2.id);
    }
    if (c1 != oldCheckedOneCategory) {
      int oIndex = 0;
      // 解决初始化为0无法自动滚回初始位置问题
      if (index == 0) {
        oIndex = 1;
      }

      _shopClassNavTabController = TabController(
          length: c1!.subs!.length,
          initialIndex: oIndex,
          vsync: ScrollableState());
      notifyListeners();
      Future.delayed(Duration(milliseconds: 300)).then((value) {
        _shopClassNavTabController!.animateTo(index);
      });
    } else {
      _shopClassNavTabController!.animateTo(index);
    }
  }

  void resetShopClassNavController() {}

  FilterItemEntity? getCheckedOneSort() {
    return _checkedOneSort;
  }

  void setCheckedSort(FilterItemEntity s1, FilterItemEntity s2) {
    _checkedOneSort = s1;
    _checkedTwoSort = s2;
    notifyListeners();
  }

  FilterItemEntity? getCheckedTwoRegion() {
    if (_checkedTwoRegion == null) {
      return FilterItemEntity(null, null, []);
    }
    return _checkedTwoRegion;
  }

  FilterItemEntity? getCheckedTwoCategory() {
    return _checkedTwoCategory;
  }

  FilterItemEntity? getCheckedTwoSort() {
    if (_checkedTwoSort == null) {
      return FilterItemEntity(null, null, []);
    }
    return _checkedTwoSort;
  }

  List<FilterItemEntity> getRegions() {
    return _regions;
  }

  List<FilterItemEntity> getCategorys() {
    return _categorys;
  }

  List<FilterItemEntity> getSorts() {
    return _sorts;
  }

  TabController? getShopClassNavTabController() {
    return _shopClassNavTabController;
  }

  int? getShopTypeNavIndex() {
    return _shopTypeNavIndex;
  }

  void setShopTypeNavIndex(int? index) {
    _shopTypeNavIndex = index;
  }

  String? getKeywords() {
    return _keywords;
  }

  void setKeywords(String s) {
    _keywords = s;
  }

  void reset() {
    if (_regions.length > 0) {
      FilterItemEntity onwItem = _regions[0];
      FilterItemEntity twoItem;
      if (onwItem.subs != null && onwItem.subs!.length > 0) {
        twoItem = onwItem.subs![0];
      } else {
        twoItem = FilterItemEntity(null, null, []);
      }
      setCheckedRegion(onwItem, twoItem);
    }

    if (_categorys.length > 0) {
      FilterItemEntity onwItem = _categorys[0];
      FilterItemEntity twoItem;
      if (onwItem.subs != null && onwItem.subs!.length > 0) {
        twoItem = onwItem.subs![0];
      } else {
        twoItem = FilterItemEntity(null, null, []);
      }
      setCheckedCategory(onwItem, twoItem);
    }

    if (_sorts.length > 0) {
      FilterItemEntity onwItem = _sorts[0];
      FilterItemEntity twoItem;
      if (onwItem.subs != null && onwItem.subs!.length > 0) {
        twoItem = onwItem.subs![0];
      } else {
        twoItem = FilterItemEntity(null, null, []);
      }
      setCheckedSort(onwItem, twoItem);
    }
  }
}
