import 'package:flutter/material.dart';
import 'package:common_module/utils/xapp_utils.dart';
import 'package:life_module/components/life_image.dart';
import 'package:life_repository/life_repository.dart';
import 'package:common_module/widget/image/index.dart';
import 'package:qr_flutter/qr_flutter.dart';
import 'package:flutter/cupertino.dart';
import 'dart:async';
import 'dart:io';
import 'dart:typed_data';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';
import 'dart:ui' as ui;
import 'package:path_provider/path_provider.dart';
import 'package:common_module/utils/permission_utils.dart';
import 'package:modal/modal.dart';
import 'package:flutter_album_save/flutter_album_save.dart';
import 'package:flutter_native_toast/flutter_native_toast.dart';
import 'package:common_module/utils/system_share_utils.dart';
import 'package:common_module/utils/clipboard_utils.dart';

class ShareDialogUtils {
  static ShareDialogUtils? _instance;
  static ShareDialogUtils? getInstance() {
    if (_instance == null) {
      _instance = new ShareDialogUtils();
    }
    return _instance;
  }

  void show({required BuildContext context, ShopInfo? shopInfo, String? firstCateId}) {
    var route = _LoadingDialogRouter(_Dialog(shopInfo, firstCateId));
    Navigator.of(context).push(route);
  }
}

class _Dialog extends StatefulWidget {
  final ShopInfo? shopInfo;
  final String? firstCateId;
  _Dialog(this.shopInfo, this.firstCateId);
  @override
  State<StatefulWidget> createState() => _DialogState();
}

class _DialogState extends State<_Dialog> {
  List<_Item> items = [
    _Item(0, "assets/download_icon.png", "保存图片"),
    _Item(1, "assets/ic_life_copy.png", "复制文案"),
    _Item(2, "assets/wechat2.png", "微信好友")
  ];
  GlobalKey globalKey = GlobalKey();
  String? shareUrl;
  ShopInfo? get shopInfo => widget.shopInfo;

  @override
  void initState() {
    super.initState();
    getShareUrl();
  }

  void getShareUrl() async {
    String? url = await ShopInfoRepository.get()
        .getShareUrl(widget.shopInfo!.shopId!, widget.firstCateId!);
    setState(() {
      shareUrl = url;
    });
  }

  @override
  Widget build(BuildContext context) {
    List<Widget> childs = items.map((e) {
      return Expanded(
        child: InkWell(
          child: Container(
            child: Column(
              children: [
                SizedBox(
                  height: 17.rpx,
                ),
                Image.asset(e.icon, width: 45.rpx, height: 45.rpx),
                SizedBox(
                  height: 11.rpx,
                ),
                Text(
                  e.name,
                  style:
                      TextStyle(color: rgba(51, 51, 51, 1), fontSize: 12.rpx),
                ),
                SizedBox(
                  height: 24.rpx,
                ),
              ],
            ),
          ),
          onTap: () async {
            if (shareUrl == null) {
              FlutterNativeToast.showToast('请稍后');
              return;
            }
            String copyText = '''店铺名称: ${shopInfo!.shopTitle}
评分: ${shopInfo!.commentScore}
人均: ${shopInfo!.avgPrice}
地址: ${shopInfo!.address}
——————————————————
$shareUrl
              ''';
            ClipboardUtils.copyText(copyText);
            if (e.id == 0 || e.id == 2) {
              File file = await widgetToImage();
              FlutterNativeToast.showToast('文案已复制');
              if (e.id == 0) {
                var _permission =
                  Platform.isIOS ? Permission.photos : Permission.storage;
              var permission = await PermissionUtils.getInstance()!
                  .has(permissions: [_permission]);
              var status = permission[_permission];

              if (status != PermissionUtilsStatus.granted) {
                bool confirm = await (Modal.showModal(
                    title: "温馨提示",
                    msg: "为给您提供更好的服务，使用此功能前需获取您的储存权限",
                    cancelBtnText: '拒绝',
                    confirmBtnText: '同意') as FutureOr<bool>);
                if (!confirm) {
                  return;
                }
                if (status == PermissionUtilsStatus.permanentlyDenied) {
                  PermissionUtils.getInstance()!.openAppSetting();
                  return;
                }
                permission = await PermissionUtils.getInstance()!
                    .request(permissions: [_permission]);
                status = permission[_permission];
                if (status != PermissionUtilsStatus.granted) {
                  return;
                }
              }
                await FlutterAlbumSave.saveImageToAlbum(file.path, '小熊有好货');
                Navigator.of(context).pop();
              } else {
                SystemShareUtils.getInstance()!.shareImage(file.path);
              }
            } else {
              Navigator.of(context).pop();
              FlutterNativeToast.showToast('文案已复制');
            }
          },
        ),
      );
    }).toList();
    return Scaffold(
      backgroundColor: rgba(0, 0, 0, 0.3),
      body: Column(
        children: [
          Spacer(),
          Container(
              alignment: Alignment.center,
              child: RepaintBoundary(
                key: globalKey,
                child: ClipRRect(
                    borderRadius: BorderRadius.circular(8.rpx),
                    child: Container(
                      width: 243.rpx,
                      padding: EdgeInsets.all(10.rpx),
                      decoration: BoxDecoration(
                        color: Colors.white,
                        // borderRadius: BorderRadius.circular(12.rpx)
                      ),
                      child: Column(
                        children: [
                          ClipRRect(
                              borderRadius: BorderRadius.circular(8.rpx),
                              child: Container(
                                height: 222.rpx,
                                width: 222.rpx,
                                child: CachedNetworkImage(
                                  imageUrl: widget.shopInfo!.shopImg!,
                                  height: 222.rpx,
                                  width: 222.rpx,
                                  fit: BoxFit.cover,
                                ),
                              )),
                          SizedBox(
                            height: 12.rpx,
                          ),
                          Container(
                            alignment: Alignment.centerLeft,
                            child: Text(
                              widget.shopInfo!.shopTitle!,
                              style: TextStyle(
                                  color: rgba(48, 38, 0, 1),
                                  fontSize: 12.rpx,
                                  height: 1.rpx,
                                  fontWeight: FontWeight.bold),
                            ),
                          ),
                          SizedBox(
                            height: 20.rpx,
                          ),
                          Container(
                            height: 84.rpx,
                            alignment: Alignment.topLeft,
                            child: Row(
                              children: [
                                Container(
                                  width: 84.rpx,
                                  height: 84.rpx,
                                  alignment: Alignment.center,
                                  decoration: BoxDecoration(
                                      border: Border.all(
                                          color: rgba(151, 151, 151, 0.5),
                                          width: 0.5.rpx)),
                                  child: Builder(
                                    builder: (context) {
                                      if (shareUrl == null) {
                                        return CupertinoActivityIndicator(
                                          radius: 12,
                                        );
                                      }
                                      return QrImage(
                                        data: shareUrl!,
                                        version: QrVersions.auto,
                                        size: 84.rpx,
                                        padding: EdgeInsets.zero,
                                      );
                                    },
                                  ),
                                ),
                                SizedBox(
                                  width: 12.rpx,
                                ),
                                Expanded(
                                  child: Column(
                                    mainAxisAlignment: MainAxisAlignment.center,
                                    children: [
                                      Container(
                                        height: 16.rpx,
                                        child: Row(
                                          children: [
                                            LifeImage(
                                              name:
                                                  "assets/images/share/ic_copy.png",
                                              width: 16.rpx,
                                              height: 16.rpx,
                                            ),
                                            SizedBox(
                                              width: 6.rpx,
                                            ),
                                            Text(
                                              "第一步：长按识别二维码",
                                              style: TextStyle(
                                                  color: rgba(102, 102, 102, 1),
                                                  fontSize: 8.rpx),
                                            )
                                          ],
                                        ),
                                      ),
                                      SizedBox(
                                        height: 8.rpx,
                                      ),
                                      Container(
                                        height: 16.rpx,
                                        child: Row(
                                          children: [
                                            LifeImage(
                                              name:
                                                  "assets/images/share/ic_copy.png",
                                              width: 16.rpx,
                                              height: 16.rpx,
                                            ),
                                            SizedBox(
                                              width: 6.rpx,
                                            ),
                                            Text(
                                              "第二步：查看优惠",
                                              style: TextStyle(
                                                  color: rgba(102, 102, 102, 1),
                                                  fontSize: 8.rpx),
                                            )
                                          ],
                                        ),
                                      ),
                                      SizedBox(
                                        height: 8.rpx,
                                      ),
                                      Container(
                                        height: 16.rpx,
                                        child: Row(
                                          children: [
                                            LifeImage(
                                              name:
                                                  "assets/images/share/ic_scan.png",
                                              width: 16.rpx,
                                              height: 16.rpx,
                                            ),
                                            SizedBox(
                                              width: 6.rpx,
                                            ),
                                            Text(
                                              "第三步：领券购买",
                                              style: TextStyle(
                                                  color: rgba(102, 102, 102, 1),
                                                  fontSize: 8.rpx),
                                            )
                                          ],
                                        ),
                                      )
                                    ],
                                  ),
                                )
                              ],
                            ),
                          ),
                        ],
                      ),
                    )),
              )),
          SizedBox(
            height: 32.rpx,
          ),
          Container(
            decoration: BoxDecoration(
                color: Colors.white,
                borderRadius:
                    BorderRadius.vertical(top: Radius.circular(16.rpx))),
            child: Column(
              children: [
                Container(
                  child: Row(
                    children: childs,
                  ),
                ),
                SafeArea(
                  top: false,
                  child: InkWell(
                    onTap: () {
                      Navigator.of(context).pop();
                    },
                    child: Container(
                      height: 49.rpx,
                      alignment: Alignment.center,
                      child: Text(
                        "取消",
                        style: TextStyle(
                            color: rgba(102, 102, 102, 1),
                            fontSize: 15.rpx,
                            height: 1.rpx),
                      ),
                    ),
                  ),
                )
              ],
            ),
          )
        ],
      ),
    );
  }

  Future<File> widgetToImage() async {
    Completer completer = Completer<File>();
    RenderRepaintBoundary render = globalKey.currentContext!.findRenderObject() as RenderRepaintBoundary;
    ui.Image image = await render.toImage(pixelRatio: 2.0);
    ByteData byteData = await (image.toByteData(format: ui.ImageByteFormat.png) as FutureOr<ByteData>);
    var imgData = byteData.buffer.asUint8List();
    String sTempDir = (await getTemporaryDirectory()).path;
    bool isDirExist = await Directory(sTempDir).exists();
    if (!isDirExist) {
      Directory(sTempDir).create();
    }
    var time = DateTime.now().millisecondsSinceEpoch;
    File file = File(sTempDir + "/widget_image_$time.png");
    file = await file.writeAsBytes(imgData,mode:FileMode.writeOnly,flush: true);
    completer.complete(file);
    return completer.future as FutureOr<File>;
  }
}

// loading路由
class _LoadingDialogRouter extends PageRouteBuilder {
  final Widget page;

  _LoadingDialogRouter(this.page)
      : super(
          opaque: false,
          barrierColor: Color(0x00000001),
          pageBuilder: (context, animation, secondaryAnimation) => page,
          transitionsBuilder: (context, animation, secondaryAnimation, child) =>
              child,
        );
}

class _Item {
  int id;
  String icon;
  String name;
  _Item(this.id, this.icon, this.name);
}
