import 'dart:convert';

import 'package:common_module/utils/navigate_utils.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:life_module/util/resource.dart';
import 'package:life_module/views/city/components/index.dart';
import 'package:azlistview/azlistview.dart';
import 'entity/city_model.dart';

class CityPage extends StatefulWidget {
  final Map<String, dynamic>? param;
  const CityPage({Key? key, this.param}) : super(key: key);

  @override
  _CityPageState createState() => _CityPageState();
}

class _CityPageState extends State<CityPage> {
  List<CityModel> citysList = List<CityModel>.empty(growable: true);

  ValueNotifier<SearchCityInfo?> listenable =
      ValueNotifier<SearchCityInfo?>(null);

  String? _currentCity = '';

  @override
  void initState() {
    super.initState();
    if (widget.param != null && widget.param!['currentCity'] != null) {
      _currentCity = widget.param!['currentCity'] as String?;
    }
    if (_currentCity!.isNotEmpty) {
      citysList.add(
        CityModel(
          type: 0,
          name: '当前城市',
          tagIndex: '0',
          citys: [_currentCity],
        ),
      );
    }
    citysList.add(
      CityModel(
        type: 1,
        name: '热门城市',
        tagIndex: '1',
        citys: ['北京', '上海', '广州', '深圳', '杭州', '南京', '重庆', '郑州', '西安', '成都'],
      ),
    );
    SearchCityInfo cityInfo = SearchCityInfo(citys: citysList, isSearch: false);
    listenable.value = cityInfo;
    rootBundle.loadString(Res.chinaCityData).then((value) {
      List list = json.decode(value);
      list.forEach((v) {
        citysList.add(CityModel.fromJson(v));
      });
    }).whenComplete(() {
      _handleList(list: citysList);
    });
  }

  void _handleList({required List<CityModel> list}) {
    if (list.isEmpty) return;
    for (int i = 0, length = list.length; i < length; i++) {
      CityModel model = list[i];
      if (model.type == -1) {
        String pinyin = model.namePinyin!;
        String tag = pinyin.substring(0, 1).toUpperCase();
        list[i].namePinyin = pinyin;
        if (RegExp('[A-Z]').hasMatch(tag)) {
          list[i].tagIndex = tag;
        } else {
          list[i].tagIndex = '#';
        }
      }
    }
    // A-Z sort.
    SuspensionUtil.sortListBySuspensionTag(list);
    // show sus tag.
    SuspensionUtil.setShowSuspensionStatus(citysList);
    SearchCityInfo cityInfo = SearchCityInfo(citys: citysList, isSearch: false);
    listenable.value = cityInfo;
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Color(0xFFF5F5F5),
      appBar: buildAppBar(title: '选择城市',onPressed:(){
        NavigateUtils.pop(arguments:{});
      }) as PreferredSizeWidget?,
      body: _buildBody(),
    );
  }

  TextEditingController _controller = TextEditingController();

  Widget _buildBody() {
    return Column(
      children: [
        //搜索框
        buildSearchWidget(
          hintText: '请输入城市名称',
          onCityTap: () {},
          controller: _controller,
          onChanged: onChangedText,
        ),
        Expanded(
          child: buildAzListViewWidget(
            notifier: listenable,
            onCityTap: (String? city) {
              NavigateUtils.pop(arguments: {'city': city});
            },
            reEnteTap: () {
              _controller.clear();
              onChangedText('');
            },
          ),
          flex: 1,
        ),
      ],
    );
  }

  void onChangedText(String text) {
    Iterable<CityModel> citys = citysList
        .where((ele) => (ele.type == 0 || ele.name!.indexOf(text) > -1));
    listenable.value = SearchCityInfo(
      isSearch: text.isNotEmpty,
      citys: List<CityModel>.from(citys),
      keywords: text,
    );
  }

  @override
  void dispose() {
    listenable.dispose();
    _controller.dispose();
    super.dispose();
  }
}
