import 'package:azlistview/azlistview.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:life_module/util/resource.dart';
import 'package:life_module/views/city/entity/city_model.dart';
import 'package:common_module/utils/xapp_utils.dart';

Widget buildAppBar({String title = '',void Function()? onPressed}) {
  return AppBar(
    backgroundColor: Colors.white,
    elevation: 0.1,
    title: Text(
      title,
      style: TextStyle(
        fontSize: 17,
        fontWeight: FontWeight.w500,
        color: Color(0xFF333333),
      ),
    ),
    leading: IconButton(
      padding: EdgeInsets.all(5),
      icon: ImageIcon(
        AssetImage('assets/web_view_back_btn.png'),
        size: 26,
      ),
      onPressed:onPressed
    ),
  );
}

///
/// 搜索布局
/// [city]            当前城市
/// [hintText]        提示信息
/// [onChanged]       搜索框文字发生改变
/// [onSubmitted]     点击提交
/// [onCityTap]       点击提交
///
Widget buildSearchWidget({
  String? city,
  String? hintText,
  void Function(String)? onChanged,
  void Function(String)? onSubmitted,
  void Function()? onCityTap,
  void Function()? onEnterCity,
  bool enabled = true,
  TextEditingController? controller,
}) {
  Widget _child = Container(
    decoration: BoxDecoration(
      color: Colors.white,
      borderRadius: BorderRadius.all(Radius.circular(24)),
    ),
    child: TextField(
      controller: controller,
      maxLines: 1,
      autofocus: false,
      enabled: enabled,
      style: TextStyle(),
      onChanged: onChanged,
      onSubmitted: onSubmitted,
      textAlign: TextAlign.left,
      decoration: InputDecoration(
        contentPadding: EdgeInsets.only(bottom: 15, left: 0),
        hintText: hintText,
        border: InputBorder.none,
        hintStyle: TextStyle(
          fontSize: 13,
          color: Color(0xFFAAAAAA),
        ),
        prefixIcon: Icon(Icons.search),
      ),
    ),
    height: 34,
  );
  if (city != null) {
    _child = Row(
      children: [
        InkWell(
          child: Container(
            decoration: BoxDecoration(
              color: Colors.white,
              borderRadius: BorderRadius.all(Radius.circular(24)),
            ),
            height: 34,
            width: 110,
            child: Row(
              mainAxisAlignment: MainAxisAlignment.center,
              children: [
                Text(
                  city,
                  style: TextStyle(
                    color: Color(0xFF302600),
                    fontSize: 13,
                    fontWeight: FontWeight.bold,
                    height: 1.2,
                  ),
                  maxLines: 1,
                  overflow: TextOverflow.ellipsis,
                ),
                Container(
                  margin: EdgeInsets.only(left: 4),
                  height: 12,
                  width: 6,
                  alignment: Alignment.centerLeft,
                  child: Image.asset(Res.icArrowGray),
                )
              ],
            ),
          ),
          onTap: onCityTap,
        ),
        SizedBox(
          width: 8,
        ),
        Expanded(child: _child, flex: 1),
      ],
    );
  }
  return InkWell(
    child: Container(
      height: 35,
      margin: EdgeInsets.symmetric(vertical: 8, horizontal: 13),
      child: _child,
    ),
    onTap: () {
      onEnterCity!();
    },
  );
}

///
/// 历史定位或者附加定位
///
///
Widget buildLocationsWidget({
  required String title,
  required Widget action,
  bool isLoading = false,
  List<dynamic>? list = const [],
  Widget? bottom,
  void Function(Map<String, dynamic>)? onAddressTap,
  EdgeInsetsGeometry padding = const EdgeInsets.symmetric(horizontal: 13),
}) {
  if (list == null || list.isEmpty) {
    return Container();
  }
  return Column(
    crossAxisAlignment: CrossAxisAlignment.start,
    children: [
      Container(
        margin: EdgeInsets.only(top: 20),
        padding: padding,
        height: 32,
        child: Row(
          children: [
            Text(
              title,
              style: TextStyle(fontSize: 13, color: Color(0xFF666666)),
            ),
            Spacer(),
            action,
          ],
        ),
      ),
      isLoading
          ? Container(
              margin: EdgeInsets.only(top: 10),
              alignment: Alignment.center,
              child: CupertinoActivityIndicator(),
            )
          : ListView.builder(
              shrinkWrap: true,
              itemCount: list.length,
              physics: NeverScrollableScrollPhysics(),
              itemBuilder: (BuildContext context, int index) {
                Map<String, dynamic> data = list[index];
                return InkWell(
                  child: Container(
                    child: Column(
                      children: [
                        Container(
                          height: 40,
                          alignment: Alignment.centerLeft,
                          margin: EdgeInsets.symmetric(horizontal: 13),
                          child: Text(
                            '${data['name']}',
                            style: TextStyle(
                                color: Color(0xFF302600), fontSize: 13),
                            overflow: TextOverflow.ellipsis,
                            maxLines: 1,
                          ),
                          decoration: BoxDecoration(
                            border: Border(
                              bottom: BorderSide(
                                color: Color(0xFFF5F5F5),
                                width: 0.5,
                              ),
                            ),
                          ),
                        )
                      ],
                    ),
                    color: Colors.white,
                  ),
                  onTap: () => onAddressTap!(data),
                );
              },
            ),
      if (bottom != null) bottom
    ],
  );
}

Widget buildCurrentLocationsWidget({String title = '', void Function()? onTap}) {
  return InkWell(
    onTap: onTap,
    child: Container(
      margin: EdgeInsets.symmetric(horizontal: 13),
      padding: EdgeInsets.symmetric(horizontal: 12),
      height: 40,
      decoration: BoxDecoration(
        color: Colors.white,
        borderRadius: BorderRadius.all(
          Radius.circular(20),
        ),
      ),
      child: Row(
        children: [
          Text(
            title,
            style: TextStyle(color: Color(0xFF302600), fontSize: 13),
          ),
          Spacer(),
          Image.asset(Res.icGpsOrg, width: 16, height: 16),
        ],
      ),
    ),
  );
}

///
/// 当前城市和热门城市
/// [citys]         城市列表
/// [current]       当前城市时显示定位
///
///
Widget buildCityWidget({
  required List<String?> citys,
  bool current = false,
  EdgeInsetsGeometry padding = const EdgeInsets.only(left: 13),
  void Function(String)? onCityTap,
}) {
  return GridView.builder(
    padding: EdgeInsets.symmetric(
      horizontal: padding.horizontal,
      vertical: 7,
    ),
    itemCount: citys.length,
    shrinkWrap: true,
    physics: NeverScrollableScrollPhysics(),
    gridDelegate: SliverGridDelegateWithMaxCrossAxisExtent(
      maxCrossAxisExtent: 100,
      crossAxisSpacing: 12,
      mainAxisSpacing: 12,
      childAspectRatio: 3.1,
    ),
    itemBuilder: (BuildContext context, int index) {
      String city = citys[index]!;
      Widget _child = Text(
        city,
        style: TextStyle(color: Color(0xFF302600), fontSize: 13),
      );
      if (current) {
        _child = Row(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            Image.asset(
              Res.icGpsOrg,
              width: 16,
              height: 16,
            ),
            SizedBox(width: 4),
            _child,
          ],
        );
      }
      return InkWell(
        child: Container(
          decoration: BoxDecoration(
            color: Colors.white,
            borderRadius: BorderRadius.all(Radius.circular(4)),
          ),
          height: 34,
          alignment: Alignment.center,
          child: _child,
        ),
        onTap: () => onCityTap != null ? onCityTap(city) : null,
      );
    },
  );
}

///
/// AZ排序ListView
/// [citysList]           城市列表数据
///
///
Widget buildAzListViewWidget({
  required ValueNotifier<SearchCityInfo?> notifier,
  required Function(String?) onCityTap,
  required Function() reEnteTap,
}) {
  return ValueListenableBuilder(
    valueListenable: notifier,
    builder: (BuildContext context, SearchCityInfo? cityInfo, Widget? child) {
      if (cityInfo!.isSearch! && cityInfo.citys!.length == 1) {
        return Column(
          children: [
            Container(
              padding: EdgeInsets.only(left: 13.0),
              width: MediaQuery.of(context).size.width,
              height: 32,
              color: Color(0xFFF5F5F5),
              alignment: Alignment.centerLeft,
              child: Text(
                '当前城市',
                style: TextStyle(fontSize: 13.0, color: Color(0xFF666666)),
              ),
            ),
            buildCityWidget(
              citys: cityInfo.citys![0].citys!,
              current: true,
              onCityTap: onCityTap,
            ),
            SizedBox(height: 120),
            Container(
              child: Text.rich(
                TextSpan(
                  children: [
                    TextSpan(
                      text: '没有找到“',
                      style: TextStyle(fontSize: 14, color: Color(0xFF666666)),
                    ),
                    TextSpan(
                      text: '${cityInfo.keywords}',
                      style: TextStyle(fontSize: 14, color: Color(0xFFFF8000)),
                    ),
                    TextSpan(
                      text: '”这个城市',
                      style: TextStyle(fontSize: 14, color: Color(0xFF666666)),
                    )
                  ],
                ),
              ),
            ),
            SizedBox(height: 12),
            InkWell(
              child: Container(
                height: 40,
                width: 160,
                decoration: BoxDecoration(
                  color: Color(0xFFFFCA00),
                  borderRadius: BorderRadius.all(Radius.circular(20)),
                ),
                alignment: Alignment.center,
                child: Text(
                  '重新输入',
                  style: TextStyle(
                    fontSize: 16,
                    fontWeight: FontWeight.w600,
                    color: Color(0xFF3D3101),
                  ),
                ),
              ),
              onTap: () => reEnteTap(),
            ),
          ],
        );
      }
      return AzListView(
        data: cityInfo.citys!,
        itemCount: cityInfo.citys!.length,
        itemBuilder: (BuildContext context, int index) {
          CityModel model = cityInfo.citys![index];
          if (model.type == 0) {
            //0当前城市
            return buildCityWidget(
              citys: model.citys!,
              current: true,
              onCityTap: onCityTap,
            );
          } else if (model.type == 1) {
            //热门城市
            return buildCityWidget(
              citys: model.citys!,
              current: false,
              onCityTap: onCityTap,
            );
          }

          Text textWidget;
          if (cityInfo.isSearch!) {
            List<TextSpan> childs = model.name!.split("").map((s) {
              bool isKeyWordChar = cityInfo.keywords.indexOf(s) > -1;
              return TextSpan(
                text: s,
                style: TextStyle(
                  color:
                      isKeyWordChar ? rgba(255, 128, 0, 1) : rgba(48, 38, 0, 1),
                ),
              );
            }).toList();

            textWidget = Text.rich(
              TextSpan(children: childs),
              style: TextStyle(fontSize: 13.rpx),
            );
          } else {
            textWidget = Text(
              model.name!,
              style: TextStyle(color: Color(0xFF302600), fontSize: 13),
            );
          }

          return InkWell(
            child: Container(
              color: Colors.white,
              margin: EdgeInsets.only(
                top: (cityInfo.isSearch! && index == 1) ? 12 : 0,
              ),
              child: Column(
                children: [
                  Container(
                    child: textWidget,
                    alignment: Alignment.centerLeft,
                    height: 40,
                    margin: EdgeInsets.only(left: 13, right: 12),
                    decoration: BoxDecoration(
                      border: Border(
                        bottom: BorderSide(color: Color(0xFF979797), width: .2),
                      ),
                    ),
                  ),
                ],
              ),
            ),
            onTap: () => onCityTap(model.name),
          );
        },
        susItemBuilder: (BuildContext context, int index) {
          CityModel model = cityInfo.citys![index];
          if (model.type == 0) {
            return Container(
              padding: EdgeInsets.only(left: 13.0),
              width: MediaQuery.of(context).size.width,
              height: 32,
              color: Color(0xFFF5F5F5),
              alignment: Alignment.centerLeft,
              child: Text(
                '当前城市',
                style: TextStyle(fontSize: 13.0, color: Color(0xFF666666)),
              ),
            );
          } else if (model.type == 1) {
            return Container(
              padding: EdgeInsets.only(left: 13.0),
              width: MediaQuery.of(context).size.width,
              height: 32,
              alignment: Alignment.centerLeft,
              color: Color(0xFFF5F5F5),
              child: Text(
                '热门城市',
                style: TextStyle(fontSize: 13.0, color: Color(0xFF666666)),
              ),
            );
          }
          String tag = model.getSuspensionTag();
          return Container(
            height: 32,
            padding: EdgeInsets.only(left: 13.0),
            color: Color(0xFFF5F5F5),
            width: MediaQuery.of(context).size.width,
            alignment: Alignment.centerLeft,
            child: Text(
              '$tag',
              softWrap: false,
              style: TextStyle(fontSize: 13.0, color: Color(0xFF666666)),
            ),
          );
        },
      );
    },
  );
}
