import 'dart:async';
import 'dart:io';

import 'package:amaps_location/location.dart';
import 'package:common_module/utils/navigate_utils.dart';
import 'package:common_module/utils/store_utils.dart';
import 'package:common_module/utils/xapp_utils.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:life_module/route/index.dart';
import 'package:life_module/util/resource.dart';
import 'package:life_repository/life_repository.dart';
import 'package:life_repository/repository/amap_repository.dart';
import 'package:common_module/utils/location_permission_utils.dart';
import 'components/index.dart';
import 'package:modal/modal.dart';
import 'package:common_module/utils/loading_dialog_utils.dart';

class LifeLocationPage extends StatefulWidget {
  final Map<String, dynamic>? param;
  const LifeLocationPage({Key? key, this.param}) : super(key: key);
  @override
  _LifeLocationPageState createState() => _LifeLocationPageState();
}

class _LifeLocationPageState extends State<LifeLocationPage> {
  ///当前城市
  String? _currentCity;
  String? _mCurrentCity;
  List<dynamic>? _historyLocations;
  //附近位置
  List<dynamic> _nearbyLocations = List<dynamic>.empty(growable: true);

  List<MapTipEntity>? _searchTips;

  LocationEntity? _currentLocationEntity;


  @override
  void initState() {
    super.initState();
    Future.delayed(Duration.zero,()=>startLocation(isInit: false,first:true));
    getHistoryLocations();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar:  buildAppBar(
        title: '选择定位地址',
        onPressed:(){
          late Map<String,dynamic> arguments;
          if(_currentLocationEntity != null){
            arguments = {
              'address': _currentLocationEntity?.address,
              'name': _currentLocationEntity?.aoiName ?? _currentLocationEntity?.poiName ?? _currentLocationEntity?.streetNum?? _currentLocationEntity?.address,
              'location': '${_currentLocationEntity?.longitude},${_currentLocationEntity?.latitude}',
              'city': _currentLocationEntity?.city
            };
          }
          NavigateUtils.pop(arguments:arguments);
        }
      ) as PreferredSizeWidget?,
      body: _buildBody(),
    );
  }

  Widget _buildBody() {
    return Column(
      children: [
        buildSearchWidget(
          enabled: !(_currentCity == null || _currentCity!.isEmpty),
          city: _currentCity ?? '加载中...',
          hintText: (_currentCity == null || _currentCity!.isEmpty)
              ? '请输入城市名称'
              : '在$_currentCity内搜索',
          onCityTap: () {
            pushCity();
          },
          onEnterCity: () {
            if (_currentCity == null || _currentCity!.isEmpty) {
              pushCity();
            }
          },
          onChanged: (value) {
            searchForKeywordsTips(keywords: value);
          },
        ),
        Expanded(
          child: (_searchTips == null || _searchTips!.isEmpty)
              ? CustomScrollView(
                  slivers: [
                    //获取当前定位按钮
                    SliverToBoxAdapter(
                      child: buildCurrentLocationsWidget(
                        onTap: () async {
                          startLocation(isInit: false, isCurrent: true);
                        },
                        title: '获取当前定位',
                      ),
                    ),
                    //历史定位
                    SliverToBoxAdapter(
                      child: buildLocationsWidget(
                        title: '历史定位',
                        list: _historyLocations,
                        action: Text(
                          (_historyLocations == null ||
                                  (_historyLocations?.length ?? 0) < 5)
                              ? '最多显示5条历史定位'
                              : '仅显示5条历史定位',
                          style:
                              TextStyle(color: Color(0xFF999999), fontSize: 12),
                        ),
                        onAddressTap: (Map<String, dynamic> address) {
                          popAddress(arguments: address);
                        },
                      ),
                    ),
                    //附近定位
                    SliverToBoxAdapter(
                      child: buildLocationsWidget(
                          title: '附近定位',
                          isLoading: startLocationLoading,
                          list: _nearbyLocations,
                          padding: EdgeInsets.only(left: 13),
                          action: InkWell(
                            child: Container(
                              padding: EdgeInsets.symmetric(
                                vertical: 8,
                                horizontal: 13,
                              ),
                              child: Image.asset(
                                'assets/images/life/ic_location_refresh.png',
                                package: 'life_module',
                                width: 16,
                                height: 16,
                              ),
                            ),
                            onTap: () {
                              startLocation(isInit: false);
                            },
                          ),
                          onAddressTap: (Map<String, dynamic> address) {
                            popAddress(arguments: address);
                          }),
                    ),
                  ],
                )
              : ListView.builder(
                  itemCount: _searchTips!.length,
                  itemBuilder: (BuildContext context, int index) {
                    MapTipEntity entity = _searchTips![index];
                    List<TextSpan> childs = entity.name!.split("").map((s) {
                      bool isKeyWordChar = this.searchKeywords.indexOf(s) > -1;
                      return TextSpan(
                        text: s,
                        style: TextStyle(
                          color: isKeyWordChar
                              ? rgba(255, 128, 0, 1)
                              : rgba(48, 38, 0, 1),
                        ),
                      );
                    }).toList();

                    Text textWidget = Text.rich(
                      TextSpan(children: childs),
                      style: TextStyle(fontSize: 13.rpx),
                    );

                    return InkWell(
                      child: Container(
                        child: Column(
                          children: [
                            Container(
                              margin: EdgeInsets.symmetric(horizontal: 13),
                              height: 43,
                              alignment: Alignment.centerLeft,
                              child: textWidget,
                              decoration: BoxDecoration(
                                color: Colors.white,
                                border: Border(
                                  bottom: BorderSide(
                                      color: Color(0xFFF5F5F5), width: 0.5),
                                ),
                              ),
                            ),
                          ],
                        ),
                        color: Colors.white,
                      ),
                      onTap: () {
                        Map<String, dynamic> arguments = {
                          'address': entity.address,
                          'city': _currentCity, //entity.city, 这里直接用当前的城市
                          'district': entity.district,
                          'adcode': entity.adcode,
                          'location': entity.location,
                          'name': entity.name
                        };
                        popAddress(arguments: arguments);
                      },
                    );
                  }),
          flex: 1,
        ),
      ],
    );
  }

  void popAddress({required Map<String, dynamic> arguments}) {
    NavigateUtils.pop(arguments: arguments).then(
      (value) {
        _historyLocations!.removeWhere(
          (item) => item['name'] == arguments['name'],
        );
        _historyLocations!.insert(0, arguments);
        if (_historyLocations!.length > 5) {
          _historyLocations!.removeAt(5);
        }
        StoreUtils.getInstance()!.setKV(
          key: Res.historyLocation,
          value: _historyLocations,
        );
      },
    );
  }

  String searchKeywords = '';

  void searchForKeywordsTips({required String keywords}) async {
    AMapRepository.get()
        .inputSearchTips(
      key: Res.aWebMapKey,
      keywords: keywords,
      city: _currentCity,
      datatype: 'poi',
      citylimit: true,
    )
        .then(
      (value) {
        this.searchKeywords = keywords;
        setState(() {
          _searchTips = value;
        });
      },
    ).catchError(
      (error) {
        print('========> error:$error');
      },
    );
  }

  void pushCity() {
    Map<String, dynamic> arguments = {};
    if (_mCurrentCity != null && _mCurrentCity!.isNotEmpty) {
      arguments['currentCity'] = _mCurrentCity;
    }
    NavigateUtils.push(
      path: RouteCityPath.LOCATION_CITY_PAGE.path(),
      arguments: arguments,
    ).then((value) {
      if (value is Map && value['city'] != null) {
        String? _city = value['city'];
        setState(() {
          _currentCity = _city;
        });
      }
    });
  }

  ///
  /// 使用高德地图SDK定位
  ///
  void startLocation({
    bool isInit = false,
    bool isCurrent = false,
    bool first = false,
  }) async {
    ///
    /// 检查是否存在权限
    /// 如果有定位权限则读取定位权限
    /// 否则使用网络定位显示城市
    ///
    bool? enable = await AmapsLocation.create()!.isOpenGPS();
    bool has = await LocationPermissionUtils.getInstance()!.hasPermission();
    if (has && enable!) {
      //有定位权限
      var loading = LoadingDialogUtils.getInstance()!.show();
      AmapsLocation.create()!
          .getLocation(
              apiKey: Platform.isIOS ? Res.aIOSMapKey : Res.aAndroidMapKey)
          .then(
        (value) async{
          LocationEntity entity = value;
          loading.close();
          if((entity?.address??'').isEmpty){
            String location = '${entity.longitude},${entity.latitude}';
            entity = await Res.convertLocation(entity:value,location:location);
          }
          _currentLocationEntity = entity;
          if ((entity?.city ?? '').isNotEmpty) {
            if (isCurrent) {
                popAddress(arguments: {
                  'address': entity.address,
                  'name': entity.aoiName ?? entity.poiName ?? entity.streetNum?? entity.address,
                  'location': '${entity.longitude},${entity.latitude}',
                  'city': entity.city
                });
            } else {
              _mCurrentCity = entity.city;
              setState(() {
                _currentCity = entity.city;
              });
              //查询附近位置信息
              queryNearbyInfo(location: '${entity.longitude},${entity.latitude}');
            }
          } else {
            ipConfigAddress();
          }
        },
      ).catchError(
        (error) {
          ipConfigAddress();
        },
      );
    } else {
      //没有定位权限
      if (isInit) {
        ipConfigAddress();
      } else {
        if (enable!) {
          //有GPS
          bool confirm = await (Modal.showModal(
              title: "温馨提示",
              msg: "为给您提供更好的服务，需获取您的位置权限",
              cancelBtnText: '拒绝',
              confirmBtnText: '同意') as FutureOr<bool>);
          if (confirm) {
            bool has =
                await LocationPermissionUtils.getInstance()!.reqPermission();
            if (has) {
              startLocation(isInit: false, isCurrent: isCurrent);
            }
          }else if(first){
            ipConfigAddress();
          }
        } else {
          //没有GPS
          bool confirm = await (Modal.showModal(
              title: "温馨提示",
              msg: "为给您提供更好的服务，需获取您的位置权限",
              cancelBtnText: '拒绝',
              confirmBtnText: '同意') as FutureOr<bool>);
          if (confirm) {
            AmapsLocation.create()!.enableGPS().then((value) {});
          }
        }
      }
    }
  }

  bool startLocationLoading = false;

  void queryNearbyInfo({required String location}) {
    setState(() {
      startLocationLoading = true;
    });
    AMapRepository.get()
        .poi2AroundSearch(key: Res.aWebMapKey, location: location, pageSize: 5)
        .then((value) {
      _nearbyLocations.clear();
      for (var item in value) {
        _nearbyLocations.add({
          'address': item.address,
          'name': item.name,
          'id': item.id,
          'location': item.location,
          'type': item.type,
          'typecode': item.typecode,
          'pname': item.pname,
          'cityname': item.cityname,
          'adname': item.adname,
          'pcode': item.pcode,
          'citycode': item.citycode,
          'adcode': item.adcode,
        });
      }
      startLocationLoading = false;
      setState(() {});
    }).catchError((error) {});
  }

  ///
  /// 使用网络定位
  ///
  void ipConfigAddress() {
    AMapRepository.get().ipConfigAddress(key: Res.aWebMapKey).then((value) {
      if (value.status == '1' && value.city != null) {
        _mCurrentCity = _currentCity;
        setState(() {
          _currentCity = value.city;
        });
      }
    }).catchError((error) {
      print('============> 高德地图网络定位异常');
    });
  }

  ///
  /// 获取缓存的历史定位
  ///
  void getHistoryLocations() {
    _historyLocations =
        StoreUtils.getInstance()!.getForKey(key: Res.historyLocation);
    if (_historyLocations == null || _historyLocations!.isEmpty) {
      _historyLocations = List<dynamic>.empty(growable: true);
    }
  }
}
