import 'package:common_module/utils/store_utils.dart';
import 'package:mvp/mvp.dart';
import '../models/model.dart';
import '../models/app_nav_model.dart';
import '../models/sec_kill_model.dart';
import '../models/mini_banner_model.dart';
import 'package:life_module/models/filter_model.dart';
import 'package:life_module/models/shop_list_model.dart';
import 'package:common_module/utils/loading_dialog_utils.dart';

class IAction extends BaseAction<Model> {
  AppNavModel _appNavModel = AppNavModel();
  SecKillModel _killModel = SecKillModel();
  MiniBannerModel _miniBannerModel = MiniBannerModel();
  FilterModel _filterModel = FilterModel();
  ShopListModel _shopListModel = ShopListModel();

  @override
  void initState() {
    _init();
  }

  void _init() async {
    setModel(Model());
    await getModel()!.findPermission();
    initLocation();
    await getAppNavModel().getAppNavList();
    await getMiniBannerModel().getActivityConfigList();
  }

  void initLocation() async{
    // 没有定位权限，用ip定位
    bool? isFirstLifeLocation = StoreUtils.getInstance()!.getForKey(key:'is_first_life_location2');
    if(isFirstLifeLocation == null){
      StoreUtils.getInstance()!.setKV(key:'is_first_life_location2', value: true);
      await getModel()!.reqPermission();
      initLocation();
    }else if (!getModel()!.getHasPermission()!) {
      getModel()!.ipConfigAddress().then((value) => initData());
    } else {
      getModel()!.getLocation().then((value){
        initData();
      });
    }
  }

  Future<void> initData() async {
    await getModel()!.findCityId();
    String? cityId = getModel()!.getCityId();
    print('===========> cityId:$cityId');
    if (cityId != null && cityId.isNotEmpty) {
      await getSecKillModel().findDisplay(cityId);
      await getFilterModel().getFilterList(cityId);
      refreshShopList();
    }
  }

  Future<void> refreshShopList() async {
    var loading = LoadingDialogUtils.getInstance()!.show();
    Future.delayed(Duration(seconds: 2)).then((value) {
      if (loading != null) {
        loading.close();
      }
    });
    try {
      getShopListModel()
          .setShopTypeNavIndex(getFilterModel().getShopTypeNavIndex());
      getShopListModel().setCityId(getModel()!.getCityId());
      getShopListModel().setLat(getModel()!.getLatitude());
      getShopListModel().setLng(getModel()!.getLongitude());
      getShopListModel()
          .setFirstCateId(getFilterModel().getCheckedOneCategory()!.id);

      getShopListModel()
          .setSecondCateId(getFilterModel().getCheckedTwoCategory()!.id ?? "");
      if (getFilterModel().getCheckedTwoRegion()!.id == null) {
        getShopListModel()
            .setRegionId(getFilterModel().getCheckedOneRegion()!.id);
      } else {
        getShopListModel()
            .setRegionId(getFilterModel().getCheckedTwoRegion()!.id);
      }
      getShopListModel().setSortType(getFilterModel().getCheckedOneSort()!.id);
      getShopListModel().clear();
      getShopListModel().scrollTop();
      await getShopListModel().refresh();
    } catch (e) {}
    if (loading != null) {
      loading.close();
    }
  }

  void refresh() async {
    String? cityId = getModel()!.getCityId();
    try {
      await getAppNavModel().getAppNavList();
      await getMiniBannerModel().getActivityConfigList();
    } catch (e) {}
    if (cityId != null && cityId.isNotEmpty) {
      try {
        await getSecKillModel().findDisplay(cityId);
      } catch (e) {}
      await getShopListModel().refresh();
    }
  }

  AppNavModel getAppNavModel() {
    return _appNavModel;
  }

  SecKillModel getSecKillModel() {
    return _killModel;
  }

  MiniBannerModel getMiniBannerModel() {
    return _miniBannerModel;
  }

  FilterModel getFilterModel() {
    return _filterModel;
  }

  ShopListModel getShopListModel() {
    return _shopListModel;
  }
}
