import 'package:flutter/material.dart';
import 'package:common_module/utils/xapp_utils.dart';
import '../actions/action.dart';
import 'package:mvp/providers/provider.dart';
import '../models/app_nav_model.dart';
import 'package:life_repository/entity/app_nav_entity.dart';
import 'package:common_module/widget/xiaoxiong_base_image_widget/widget.dart';
import 'package:life_module/utils/activity_config_tap_utils.dart';

/// 钻石位
class DiamondBit extends StatelessWidget {
  final IAction? iAction;
  final ValueNotifier<double> scrollIndex = ValueNotifier(0.0);

  DiamondBit({Key? key, this.iAction}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return SliverToBoxAdapter(
      child: QMProvider<AppNavModel>.value(
          model: iAction!.getAppNavModel(),
          builderWidget: (context, model, child) {
            if (model.getList().length == 0) {
              return SizedBox();
            }
            return Container(
              child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: <Widget>[
                    NotificationListener(
                      onNotification: (ScrollNotification notification) {
                        double extentBefore = notification.metrics.extentBefore;
                        double extentAfter = notification.metrics.extentAfter;
                        double width = extentAfter + extentBefore;
                        scrollIndex.value = (extentBefore / width) * 10.rpx;
                        return true;
                      },
                      child: _DiamondGrid(model.getList()),
                    ),
                    ValueListenableBuilder<double>(
                      valueListenable: scrollIndex,
                      builder:
                          (BuildContext context, dynamic value, Widget? child) {
                        return _DiamondBitWidgetPagination(
                            offset: value, itemCount: model.getList().length);
                      },
                    ),
                  ]),
            );
          }),
    );
  }
}

/// 宫格
class _DiamondGrid extends StatelessWidget {
  final List<AppNavEntity> items;

  _DiamondGrid(this.items);
  @override
  Widget build(BuildContext context) {
    int itemCount = items.length;
    bool isOneRow = itemCount < 10;
    bool isFull = itemCount == 10 || itemCount == 5;
    return Theme(
      data: ThemeData(platform: TargetPlatform.macOS),
      child: Container(
        height: (isOneRow ? 78.rpx : 156.rpx) + (isFull ? 20.rpx : 0),
        padding: EdgeInsets.only(top: 8.rpx, bottom: isFull ? 20.rpx : 0),
        child: GridView.builder(
          scrollDirection: Axis.horizontal,
          gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
            crossAxisCount: isOneRow ? 1 : 2,
            crossAxisSpacing: 0,
            mainAxisSpacing: 0,
            childAspectRatio: .95,
          ),
          itemBuilder: (context, index) {
            var item = items[index];
            return InkWell(
              onTap: () {
                AppTapConfigUtils.getInstance()!.activityConfigTap(item.toJson());
              },
              child: Container(
                alignment: Alignment.topCenter,
                child: Column(
                  children: <Widget>[
                    SizedBox(
                      height: 8.rpx,
                    ),
                    Container(
                      width: 40.rpx,
                      height: 40.rpx,
                      child: XiaoxiongBaseImageWidget(
                        imageUrl: item.iconUrl!,
                        width: 40.rpx,
                        height: 40.rpx,
                        fit: BoxFit.cover,
                      ),
                    ),
                    SizedBox(
                      height: 2.rpx,
                    ),
                    Text(
                      item.name!,
                      overflow: TextOverflow.ellipsis,
                      style: TextStyle(
                          height: 1.3846153846153,
                          fontSize: 12.rpx,
                          color: rgba(48, 38, 0, 1)),
                    )
                  ],
                ),
              ),
            );
          },
          itemCount: items.length,
        ),
      ),
    );
  }
}

/// 指示器
class _DiamondBitWidgetPagination extends StatelessWidget {
  final double? offset;
  final int? itemCount;

  _DiamondBitWidgetPagination({Key? key, this.offset, this.itemCount})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    if (itemCount == 10 || itemCount! <= 5) {
      return SizedBox();
    }
    return Container(
        width: double.infinity,
        alignment: Alignment.center,
        padding: EdgeInsets.only(bottom: 9.rpx, top: 4.rpx),
        child: Container(
            width: 20.rpx,
            height: 3.rpx,
            decoration: BoxDecoration(
                color: Color(0xFFE9E9E9),
                borderRadius: BorderRadius.all(Radius.circular(2.rpx))),
            child: Stack(
              children: [
                Positioned(
                    left: offset,
                    child: Container(
                        width: 10.rpx,
                        height: 3.rpx,
                        decoration: BoxDecoration(
                          color: Color(0XFFFF8000),
                          borderRadius:
                              BorderRadius.all(Radius.circular(2.rpx)),
                        )))
              ],
            )));
  }
}
