import 'package:flutter/material.dart';
import 'package:common_module/utils/xapp_utils.dart';
import 'package:life_module/components/shop_type_nav.dart';
import './diamond_bit.dart';
import './sec_kill.dart';
import './mini_banner.dart';
import 'package:life_module/components/filter_area.dart';
import 'package:life_module/components/shop_class_nav.dart';
import 'package:life_module/components/shop_list.dart';
import 'package:life_module/components/persistent_header_delegate.dart';
import '../actions/action.dart';
import 'package:flutter/rendering.dart';
import 'package:common_module/widget/pull_widget/widget.dart';
import 'package:common_module/utils/navigate_utils.dart';
import 'package:life_module/route/index.dart';

class ScrollBody extends StatefulWidget {
  final IAction? iAction;
  final String? isAuditMode;
  final Function(double offset)? onScroll;
  final ScrollController? controller;
  ScrollBody({this.onScroll, this.iAction, this.controller,this.isAuditMode});
  @override
  State<StatefulWidget> createState() => _ScrollBodyState();
}

class _ScrollBodyState extends State<ScrollBody> {
  ScrollController? get controller => widget.controller;
  void initState() {
    super.initState();
    controller!.addListener(() {
      widget.onScroll!(controller!.offset);
    });
  }

  @override
  Widget build(BuildContext context) {
    return PullWidget(
      headerInsertIndex: 1,
      controller: widget.iAction!.getShopListModel().getRefreshController(),
      onLoad: () {
        widget.iAction!.getShopListModel().getData();
      },
      onRefresh: () {
        widget.iAction!.getShopListModel().refresh();
      },
      child: CustomScrollView(
        controller: controller,
        slivers: [
          SliverAppBar(
            backgroundColor: Colors.transparent,
            toolbarHeight: 44.rpx,
          ),
          DiamondBit(
            iAction: widget.iAction,
          ),
          SecKill(
            iAction: widget.iAction,
          ),
          MiniBanner(
            model: widget.iAction!.getMiniBannerModel(),
          ),
          SliverPersistentHeader(
              pinned: false,
              delegate: PersistentHeaderDelegate(
                  minHeight: 37.rpx,
                  maxHeight: 37.rpx,
                  child: ShopTypeNav(
                    model: widget.iAction!.getFilterModel(),
                    onChange: () {
                      widget.iAction!.refreshShopList();
                    },
                  ))),
          SliverPersistentHeader(
              pinned: true,
              delegate: PersistentHeaderDelegate(
                  minHeight: 48.rpx,
                  maxHeight: 48.rpx,
                  child: FilterArea(
                    topOffset: 81.rpx - 37.rpx,
                    model: widget.iAction!.getFilterModel(),
                    onTap: (c) {
                      var _context =
                          widget.iAction!.getShopListModel().getContext()!;
                      RenderSliver? renderSliver = _context
                          .findAncestorRenderObjectOfType<RenderSliver>();
                      if (renderSliver != null &&
                          renderSliver.constraints.overlap <= 0) {
                        Scrollable.ensureVisible(_context);
                      }
                    },
                    onChange: () {
                      widget.iAction!.refreshShopList();
                    },
                  ))),
          SliverPersistentHeader(
              pinned: true,
              delegate: PersistentHeaderDelegate(
                  minHeight: 40.rpx,
                  maxHeight: 40.rpx,
                  child: ShopClassNav(
                    model: widget.iAction!.getFilterModel(),
                    onChange: () {
                      widget.iAction!.refreshShopList();
                    },
                  ))),
          
          ShopList(
            model: widget.iAction!.getShopListModel(),
            onReset: () {
              widget.iAction!.getFilterModel().reset();
              widget.iAction!.refreshShopList();
            },
            onTap: (item) {
              NavigateUtils.push(
                path: LIFE_SHOP_PAGE,
                arguments: {
                  'isAuditMode':widget.isAuditMode,
                  'shopId': item.combo != null
                      ? item.combo!.shopId
                      : item.restaurant!.shopId,
                  'lat': widget.iAction!.getModel()!.getLatitude(),
                  'lng': widget.iAction!.getModel()!.getLongitude(),
                  'firstCateId': widget.iAction!
                      .getFilterModel()
                      .getCheckedOneCategory()!
                      .id,
                  'itemId': item.combo != null ? item.combo!.dealId : "",
                  "cityName": widget.iAction!.getModel()!.getCity()
                },
                isNative: false,
              );
            },
          )
        ],
      ),
    );
  }
}
