import 'package:flutter/material.dart';
import 'package:common_module/utils/xapp_utils.dart';
import 'package:life_module/components/search_bar.dart';
import 'package:life_module/components/life_image.dart';

import 'package:common_module/utils/navigate_utils.dart';
import 'package:life_module/route/index.dart';
import 'package:life_module/views/life/models/model.dart';
import 'package:mvp/mvp.dart';
import '../actions/action.dart';
import 'package:common_module/utils/map_utils.dart';

class SearchArea extends StatefulWidget {
  final IAction? iAction;
  final ValueNotifier<double>? scrollOffset;
  SearchArea({this.scrollOffset, this.iAction});
  @override
  State<StatefulWidget> createState() => _SearchAreaState();
}

class _SearchAreaState extends State<SearchArea> {
  @override
  Widget build(BuildContext context) {
    // 最大滚动范围
    double maxOffset = 44.rpx;
    // 离左侧最大距离
    double maxLeft = 101.rpx;

    return Container(
      child: QMProvider<Model>.value(
          model: widget.iAction!.getModel()!,
          builderWidget: (context, model, child) {
            return ValueListenableBuilder<double>(
                valueListenable: widget.scrollOffset!,
                builder: (BuildContext context, dynamic offset, Widget? child) {
                  double opacity = 1;

                  if (offset >= maxOffset) {
                    // 超出最大滚动范围
                    opacity = 0.0;
                    offset = maxOffset;
                  } else if (offset > 0) {
                    // 滚动范围>0
                    opacity = 1 - offset / maxOffset as double;
                  } else if (offset < 0) {
                    // 滚动范围小于0
                    offset = 0.0;
                  }
                  double left = offset * (maxLeft / maxOffset) * 1.5;
                  if (left > maxLeft) {
                    left = maxLeft;
                  }

                  List<Widget> items = [
                    Opacity(
                      opacity: opacity,
                      child: _Locaion(
                        iAction: widget.iAction,
                      ),
                    ),
                    Positioned(
                      top: 44.rpx,
                      left: 0,
                      right: 0,
                      child: Container(
                        height: 44.rpx,
                        child: InkWell(
                          child: SearchBar(
                            enabled: false,
                            inputColor: rgba(245, 245, 245, 1),
                          ),
                          onTap: () {
                            NavigateUtils.push(
                              path: RouteCityPath.LIFE_SEARCH_PAGE.path(),
                              arguments: {
                                'cityId': widget.iAction!.getModel()!.getCityId(),
                                'latitude':
                                    widget.iAction!.getModel()!.getLatitude(),
                                'longitude':
                                    widget.iAction!.getModel()!.getLongitude(),
                                'cityName': widget.iAction!.getModel()!.getCity(),
                              },
                              isNative: false,
                            );
                          },
                        ),
                        margin: EdgeInsets.only(left: left),
                        padding: EdgeInsets.symmetric(
                            horizontal: 13.rpx, vertical: 5.rpx),
                        transform: Matrix4.translationValues(0, -offset, 0),
                      ),
                    ),
                  ];

                  if (!model.getHasPermission()!) {
                    items.add(Positioned(
                      top: 37.rpx,
                      right: 13.rpx,
                      child: opacity <= 0
                          ? SizedBox()
                          : Opacity(
                              opacity: opacity,
                              child: _Bubble(widget.iAction),
                            ),
                    ));
                  }
                  return Container(
                    height: 88.rpx,
                    child: Stack(
                      children: items,
                    ),
                  );
                });
          }),
    );
  }
}

/// 定位按钮
class _Locaion extends StatelessWidget {
  final IAction? iAction;
  _Locaion({this.iAction});
  @override
  Widget build(BuildContext context) {
    return QMProvider<Model>.value(
        model: iAction!.getModel()!,
        builderWidget: (context, model, child) {
          return InkWell(
            child: Container(
              height: 44.rpx,
              padding: EdgeInsets.only(right: 13.rpx),
              alignment: Alignment.centerRight,
              child: Row(
                mainAxisAlignment: MainAxisAlignment.end,
                children: [
                  Image.asset('assets/ic_location.png',
                      width: 20.rpx, height: 20.rpx, fit: BoxFit.contain),
                  SizedBox(
                    width: 6.rpx,
                  ),
                  Container(
                    constraints: BoxConstraints(maxWidth: 150.rpx),
                    child: Text(
                      model.getAddres(),
                      overflow: TextOverflow.clip,
                      maxLines: 1,
                      style: TextStyle(
                          color: rgba(48, 38, 0, 1),
                          fontSize: 13.rpx,
                          height: 1.1,
                          fontWeight: FontWeight.bold),
                    ),
                  ),
                  SizedBox(
                    width: 3.rpx,
                  ),
                  Image.asset(
                    "assets/setup_enter.png",
                    height: 10.rpx,
                    color: rgba(48, 38, 0, 1),
                    width: 6.rpx,
                    fit: BoxFit.fitWidth,
                  ),
                  SizedBox(
                    width: 7.rpx,
                  )
                ],
              ),
            ),
            onTap: () async {
              // 点击这里 返回不判断权限
              model.setIsClickCitySelect(true);
              var result = await NavigateUtils.push(
                path: LIFE_LOCATION_PAGE,
                arguments: {},
                isNative: false,
              );
              if (result == null) {
                return;
              }
              var location = result['location'].split(',');
              var _latitude = '${location[1]}';
              var _longitude = '${location[0]}';
              List<num> arr = MapUtils.getInstance()!.gcj02ToGps84(
                  double.parse(_latitude), double.parse(_longitude));
              _latitude = '${arr[0]}';
              _longitude = '${arr[1]}';
              model.setLocationInfo(result['cityname'] ?? result['city'],
                  result['name'], _latitude, _longitude);
              await iAction!.initData();
              iAction!.getModel()!.findPermission();
            },
          );
        });
  }
}

/// 气泡
class _Bubble extends StatelessWidget {
  final IAction? iAction;
  _Bubble(this.iAction);
  @override
  Widget build(BuildContext context) {
    return Container(
        alignment: Alignment.center,
        height: 38.rpx,
        width: 200.rpx,
        child: Column(
          children: [
            Container(
              height: 8.rpx,
              alignment: Alignment.topRight,
              padding: EdgeInsets.only(right: 28.rpx, top: 2.5.rpx),
              child: LifeImage(
                name: "assets/images/life/ic_arrow_up.png",
                color: rgba(0, 0, 0, 0.70),
                width: 8.rpx,
                height: 9.rpx,
                fit: BoxFit.fitHeight,
              ),
            ),
            Container(
              height: 30.rpx,
              decoration: BoxDecoration(
                color: rgba(0, 0, 0, 0.70),
                borderRadius: BorderRadius.circular(25.rpx),
              ),
              child: Row(
                mainAxisAlignment: MainAxisAlignment.center,
                children: [
                  InkWell(
                      onTap: () async {
                        await iAction!.getModel()!.reqPermission();
                        if (iAction!.getModel()!.getHasPermission()!) {
                          iAction!.initLocation();
                        }
                      },
                      child: Container(
                        height: 30.rpx,
                        padding: EdgeInsets.only(
                          left: 15.rpx,
                        ),
                        alignment: Alignment.center,
                        child: Text(
                          "开启定位查看附近精准攻略",
                          style: TextStyle(
                              color: Colors.white,
                              fontSize: 12.rpx,
                              height: 1.2),
                        ),
                      )),
                  InkWell(
                    child: Container(
                      height: 30.rpx,
                      padding: EdgeInsets.only(right: 10.rpx, left: 6.rpx),
                      child: Icon(
                        Icons.close,
                        color: Colors.white,
                        size: 15.rpx,
                      ),
                    ),
                    onTap: () {
                      iAction!.getModel()!.setPermission();
                    },
                  )
                ],
              ),
            )
          ],
        ));
  }
}
