import 'package:flutter/material.dart';
import 'dart:async';
import 'package:common_module/utils/xapp_utils.dart';
import 'package:life_module/components/life_image.dart';
import 'package:life_module/views/life/actions/action.dart';
import 'package:mvp/mvp.dart';
import '../models/sec_kill_model.dart';
import 'package:life_repository/entity/sec_kill_goods.dart';
import 'package:life_repository/entity/sec_kill_session.dart';
import 'package:common_module/utils/date_time_helper.dart';
import 'package:common_module/widget/xiaoxiong_base_image_widget/widget.dart';
import 'package:flutter/cupertino.dart';
import 'package:common_module/utils/navigate_utils.dart';
import 'package:life_module/route/index.dart';

const SECOND = 1000;
const MINUTE = 60 * SECOND;
const HOUR = 60 * MINUTE;
const DAY = 24 * HOUR;

class SecKill extends StatefulWidget {
  final IAction? iAction;
  SecKill({this.iAction});
  @override
  State<StatefulWidget> createState() => _SecKillState();
}

class _SecKillState extends State<SecKill> {
  ValueNotifier<int> scrollIndex = ValueNotifier<int>(0);
  @override
  Widget build(BuildContext context) {
    return SliverToBoxAdapter(
      child: QMProvider<SecKillModel>.value(
          model: widget.iAction!.getSecKillModel(),
          builderWidget: (context, model, child) {
            if (!model.isShow() && model.getSeckillSessionList().length == 0) {
              return SizedBox();
            }

            return InkWell(
                onTap: () {
                  NavigateUtils.push(
                    path: RouteCityPath.LIFE_SEC_KILL_PAGE.path(),
                    arguments: {
                      'tid': '',
                      'times': model.getSeckillSessionList(),
                      "cityId": widget.iAction!.getModel()!.getCityId(),
                      "lat": widget.iAction!.getModel()!.getLatitude(),
                      "lng": widget.iAction!.getModel()!.getLongitude(),
                      "cityName": widget.iAction!.getModel()!.getCity()
                    },
                    isNative: false,
                  );
                },
                child: Container(
                  height: 154.rpx,
                  margin:
                      EdgeInsets.only(left: 13.rpx, top: 8.rpx, right: 13.rpx),
                  decoration: BoxDecoration(
                      color: Colors.white,
                      borderRadius: BorderRadius.circular(12.rpx)),
                  child: Column(
                    children: [
                      _SecKillTitle(
                        key: ValueKey(model
                            .getSeckillSessionList()
                            .map((e) => e.id)
                            .join("_${widget.iAction!.getModel()!.getLatitude()}_${widget.iAction!.getModel()!.getLongitude()}")),
                        seckillSessions: model.getSeckillSessionList(),
                        onNext: (seckillSession) async{
                          await widget.iAction!.getModel()!.findCityId();
                          print('==============>_cityId:${widget.iAction!.getModel()!.getCityId()}');
                          scrollIndex.value = 0;
                          model.loadSecKillGoodsByShowId(
                              widget.iAction!.getModel()!.getCityId()!,
                              seckillSession!.id!,
                              widget.iAction!.getModel()!.getLatitude(),
                              widget.iAction!.getModel()!.getLongitude());
                        },
                      ),
                      _SecKillGoodsBody(
                        model: model,
                        onChange: (index) {
                          scrollIndex.value = index;
                        },
                      ),
                      _SecKillPagination(
                        scrollIndex: scrollIndex,
                        model: model,
                      ),
                      SizedBox(
                        height: 5.rpx,
                      )
                    ],
                  ),
                ));
          }),
    );
  }
}

/// 秒杀标题
class _SecKillTitle extends StatefulWidget {
  final List<SeckillSession?>? seckillSessions;
  final Function(SeckillSession? session)? onNext;
  _SecKillTitle({this.seckillSessions, this.onNext, Key? key}) : super(key: key);

  @override
  State<StatefulWidget> createState() => _SecKillTitleState();
}

class _SecKillTitleState extends State<_SecKillTitle> {
  List<SeckillSession?>? get seckillSessions => widget.seckillSessions;

  String statusText = '';
  int? startTime;
  int? endTime;
  bool isStart = false;
  SeckillSession? seckillSession;
  @override
  void initState() {
    super.initState();
    init();
  }

  void setStartAndEndTime(SeckillSession session) {
    var timeArr = session.text!.split("-");
    String nowDate = DateUtil.formatDate(DateTime.now(), format: "yyyy-MM-dd");
    startTime = DateUtil.getDateMsByTimeStr("$nowDate ${timeArr[0]}");
    endTime = DateUtil.getDateMsByTimeStr("$nowDate ${timeArr[1]}");
  }

  void init() {
    int nowTime = DateTime.now().millisecondsSinceEpoch;
    seckillSession = seckillSessions!.firstWhere((session) {
      setStartAndEndTime(session!);
      if (startTime! <= nowTime && endTime! >= nowTime) {
        isStart = true;
        return true;
      }

      if (startTime! > nowTime) {
        isStart = false;
        return true;
      }
      startTime = 0;
      endTime = 0;
      return false;
    });
    if (seckillSession == null) {
      return;
    }
    if (startTime! > nowTime) {
      runStartTimeDown();
    } else if (startTime! < nowTime && endTime! > nowTime) {
      runEndTimeDown();
    }
    widget.onNext!(seckillSession);
  }

  void next() {
    int index = seckillSessions!.indexOf(seckillSession);
    if (index >= seckillSessions!.length - 1) {
      return;
    }
    seckillSession = seckillSessions![index + 1];
    setStartAndEndTime(seckillSession!);
    int nowTime = DateTime.now().millisecondsSinceEpoch;
    if (startTime! > nowTime) {
      isStart = false;
      runStartTimeDown();
    } else if (startTime! < nowTime && endTime! > nowTime) {
      isStart = true;
      runEndTimeDown();
    }
    widget.onNext!(seckillSession);
  }

  void runStartTimeDown() {
    int nowTime = DateTime.now().millisecondsSinceEpoch;
    int dur = startTime! - nowTime;
    int h = ((dur % DAY) / HOUR).floor();
    int i = ((dur % HOUR) / MINUTE).floor();
    int s = ((dur % MINUTE) / SECOND).floor();
    if (h <= 0 && i <= 0 && s <= 0) {
      setState(() {
        isStart = true;
      });
      runEndTimeDown();
      return;
    }

    String ht = h < 10 ? "0$h" : "$h";
    String it = i < 10 ? "0$i" : "$i";
    String st = s < 10 ? "0$s" : "$s";

    setState(() {
      statusText = "$ht:$it:$st";
    });

    Future.delayed(Duration(seconds: 1)).then((value) => runStartTimeDown());
  }

  void runEndTimeDown() {
    int nowTime = DateTime.now().millisecondsSinceEpoch;
    int dur = endTime! - nowTime;
    int h = ((dur % DAY) / HOUR).floor();
    int i = ((dur % HOUR) / MINUTE).floor();
    int s = ((dur % MINUTE) / SECOND).floor();
    if (h <= 0 && i <= 0 && s <= 0) {
      setState(() {
        statusText = "已结束";
      });
      Future.delayed(Duration(seconds: 1)).then((value) {
        next();
      });

      return;
    }

    String ht = h < 10 ? "0$h" : "$h";
    String it = i < 10 ? "0$i" : "$i";
    String st = s < 10 ? "0$s" : "$s";

    setState(() {
      statusText = "$ht:$it:$st";
    });

    Future.delayed(Duration(seconds: 1)).then((value) => runEndTimeDown());
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      height: 52.rpx,
      padding: EdgeInsets.symmetric(vertical: 16.rpx, horizontal: 8.rpx),
      child: Row(
        children: [
          LifeImage(
            name: "assets/images/life/ic_miaosha.png",
            height: 20.rpx,
            width: 76.rpx,
          ),
          SizedBox(
            width: 13.rpx,
          ),
          Container(
            height: 16.rpx,
            width: 90.rpx,
            padding: EdgeInsets.only(left: 40.rpx),
            alignment: Alignment.center,
            decoration: BoxDecoration(
              image: DecorationImage(
                  image: AssetImage(
                      "assets/images/life/ic_miaosha_time${!isStart ? '_s' : ''}.png",
                      package: "life_module")),
            ),
            child: Text(
              statusText,
              style: TextStyle(
                  color: rgba(255, 4, 0, 1),
                  fontSize: 10.rpx,
                  fontWeight: FontWeight.bold),
            ),
          ),
          Spacer(),
          Container(
            child: Row(
              children: [
                Text(
                  "全部场次",
                  style: TextStyle(
                      color: rgba(102, 102, 102, 1),
                      fontSize: 12.rpx,
                      height: 1.2),
                ),
                SizedBox(
                  width: 4.rpx,
                ),
                LifeImage(
                  name: "assets/images/life/ic_miaosha_arrow_right.png",
                  width: 8.rpx,
                  height: 8.rpx,
                )
              ],
            ),
          )
        ],
      ),
    );
  }
}

class _SecKillGoodsBody extends StatelessWidget {
  final SecKillModel? model;
  final Function(int)? onChange;
  _SecKillGoodsBody({this.model, this.onChange});
  @override
  Widget build(BuildContext context) {
    List<SecKillGoods> list = model!.getSecKillGoodsList();
    if (list.length == 0) {
      return Expanded(
        child: Container(
          alignment: Alignment.center,
          child: CupertinoActivityIndicator(radius: 10.rpx),
        ),
      );
    }
    int length = (list.length / 2).ceil();
    List<Widget> items = List.generate(length, (index) {
      int limit = 2;
      int offset = index * limit;
      int size = offset + limit;
      if (size > list.length) {
        size = list.length;
      }

      var l = list.sublist(offset, size);
      return _SecKillItem(
        list: l,
      );
    });

    return Expanded(
      child: NotificationListener(
        child: PageView(
          onPageChanged: onChange,
          children: items,
        ),
        onNotification: (dynamic e) {
          return true;
        },
      ),
    );
  }
}

/// 单场秒杀
class _SecKillItem extends StatefulWidget {
  final List<SecKillGoods>? list;
  _SecKillItem({this.list});

  @override
  State<StatefulWidget> createState() => _SecKillItemState();
}

class _SecKillItemState extends State<_SecKillItem> {
  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        padding: EdgeInsets.symmetric(horizontal: 8.rpx),
        child: Column(
          children: [
            _SecKillGoods(
              list: widget.list,
            ),
          ],
        ));
  }
}

/// 秒杀商品
class _SecKillGoods extends StatelessWidget {
  final List<SecKillGoods>? list;
  _SecKillGoods({this.list});
  @override
  Widget build(BuildContext context) {
    int len = list!.length - 1;
    List<Widget> childs = List.generate(2, (index) {
      if (index > len) {
        return Expanded(child: SizedBox());
      }
      return _SecKillGoodsItem(list![index]);
    }).toList();

    return Row(
      children: childs,
    );
  }
}

/// 单个秒杀商品
class _SecKillGoodsItem extends StatelessWidget {
  final SecKillGoods item;
  _SecKillGoodsItem(this.item);
  @override
  Widget build(BuildContext context) {
    return Expanded(
      child: Container(
        child: Row(
          children: [
            ClipRRect(
                borderRadius: BorderRadius.circular(4.rpx),
                child: Container(
                  height: 73.rpx,
                  width: 73.rpx,
                  child: XiaoxiongBaseImageWidget(
                    imageUrl: item.itemPic!,
                    height: 73.rpx,
                    width: 73.rpx,
                    fit: BoxFit.cover,
                  ),
                )),
            SizedBox(
              width: 6.rpx,
            ),
            Expanded(
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Container(
                    height: 32.rpx,
                    child: Text(
                      item.itemTitle!,
                      overflow: TextOverflow.ellipsis,
                      maxLines: 2,
                      style: TextStyle(
                          color: rgba(48, 38, 0, 1),
                          fontSize: 12.rpx,
                          height: 1.1),
                    ),
                  ),
                  Container(
                    padding: EdgeInsets.symmetric(
                        horizontal: 4.rpx, vertical: 2.rpx),
                    decoration: BoxDecoration(
                        border:
                            Border.all(color: rgba(255, 128, 0, 1), width: 0.5),
                        borderRadius: BorderRadius.only(
                            topLeft: Radius.circular(9.rpx),
                            topRight: Radius.circular(9.rpx),
                            bottomRight: Radius.circular(9.rpx))),
                    child: Text(
                      item.userCommissionText!,
                      style: TextStyle(
                          color: rgba(255, 128, 0, 1),
                          fontSize: 11.rpx,
                          height: 1.2),
                    ),
                  ),
                  Container(
                    height: 22.rpx,
                    alignment: Alignment.bottomLeft,
                    child: Row(
                      crossAxisAlignment: CrossAxisAlignment.baseline,
                      children: [
                        Text(
                          "¥",
                          style: TextStyle(
                            color: rgba(255, 4, 0, 1),
                            fontSize: 11.rpx,
                          ),
                        ),
                        Text(
                          item.endPrice!,
                          style: TextStyle(
                            color: rgba(255, 4, 0, 1),
                            fontSize: 14.rpx,
                          ),
                        ),
                        SizedBox(
                          width: 4.rpx,
                        ),
                        Text(
                          "¥${item.originalPrice}",
                          style: TextStyle(
                              color: rgba(176, 176, 176, 1),
                              fontSize: 10.rpx,
                              decoration: TextDecoration.lineThrough),
                        )
                      ],
                    ),
                  )
                ],
              ),
            )
          ],
        ),
      ),
    );
  }
}

/// 指示器
class _SecKillPagination extends StatelessWidget {
  final SecKillModel? model;
  final ValueNotifier<int>? scrollIndex;
  _SecKillPagination({this.scrollIndex, this.model});
  @override
  Widget build(BuildContext context) {
    if (model!.getSecKillGoodsList().length == 0) {
      return SizedBox(
        height: 4.rpx,
      );
    }
    return ValueListenableBuilder<int>(
        valueListenable: scrollIndex!,
        builder: (BuildContext context, dynamic offset, Widget? child) {
          List<Widget> items = List.generate(
              (model!.getSecKillGoodsList().length / 2).ceil(), (index) {
            Color color = rgba(216, 216, 216, 1);
            if (index == scrollIndex!.value) {
              color = rgba(255, 128, 0, 1);
            }
            return Container(
              width: 4.rpx,
              height: 4.rpx,
              margin: EdgeInsets.symmetric(horizontal: 2.rpx),
              decoration: BoxDecoration(
                  color: color, borderRadius: BorderRadius.circular(2.rpx)),
            );
          });
          return Container(
            height: 4.rpx,
            alignment: Alignment.center,
            child: Row(
              mainAxisAlignment: MainAxisAlignment.center,
              children: items,
            ),
          );
        });
  }
}
