import 'dart:async';

import 'package:common_module/utils/map_utils.dart';
import 'package:mvp/mvp.dart';
import 'package:life_repository/life_repository.dart';
import 'package:life_module/util/resource.dart';
import 'package:common_module/utils/amap_utils.dart';
import 'package:modal/modal.dart';

class Model extends BaseModel {
  bool? _hasPermission = false;
  String? _address = '';
  String? _city = '';
  String? _cityId = '';
  String _latitude = '';
  String _longitude = '';
  bool _isClickCitySelect = false;

  Future<void> findCityId() async {
    _cityId = await CityRepository.get().getCityId(_city!);
  }

  Future<void> getLocation() async {
    LocationEntity location = await AmapUtils.getInstance()!.getLocation();
    if((location?.address??'').isEmpty){
      String myLocation = '${location.longitude},${location.latitude}';
      location = await Res.convertLocation(entity:location,location:myLocation);
    }
    _latitude = location.latitude.toString();
    _longitude = location.longitude.toString();
    List<num> arr = MapUtils.getInstance()!
        .gcj02ToGps84(double.parse(_latitude), double.parse(_longitude));
    _latitude = '${arr[0]}';
    _longitude = '${arr[1]}';
    _city = location.city;
    _address = location.aoiName;
    notifyListeners();
  }

  Future<void> ipConfigAddress() async {
    var res = await AMapRepository.get().ipConfigAddress(key: Res.aWebMapKey);

    List<String> rectangles = res.rectangle!.split(new RegExp(r";|,"));
    if (rectangles.length > 1) {
      _latitude = rectangles[1];
      _longitude = rectangles[0];
      List<num> arr = MapUtils.getInstance()!
          .gcj02ToGps84(double.parse(_latitude), double.parse(_longitude));
      _latitude = '${arr[0]}';
      _longitude = '${arr[1]}';
    }
    _city = res.city;
    _address = _city;
    notifyListeners();
  }

  Future<void> findPermission() async {
    _hasPermission = await AmapUtils.getInstance()!.hasPermission();
    if (_hasPermission!) {
      _hasPermission = await AmapUtils.getInstance()!.isOpenGPS;
    }
    notifyListeners();
  }

  Future<void> reqPermission() async {
    bool confirm = await (Modal.showModal(
        title: "温馨提示",
        msg: "为给您提供更好的服务，需获取您的位置权限",
        cancelBtnText: '拒绝',
        confirmBtnText: '同意') as FutureOr<bool>);
    if (!confirm) {
      return;
    }
    _hasPermission = await AmapUtils.getInstance()!.reqPermission();

    if (_hasPermission!) {
      _hasPermission = await AmapUtils.getInstance()!.isOpenGPS;
      if (!_hasPermission!) {
        confirm = await (Modal.showModal(
            title: "温馨提示",
            msg: "为给您提供更好的服务，请打开GPS",
            cancelBtnText: '拒绝',
            confirmBtnText: '同意') as FutureOr<bool>);
        if (!confirm) {
          return;
        }
        _hasPermission = await AmapUtils.getInstance()!.enableGPS();
      }
    }
    notifyListeners();
  }

  void setPermission() {
    _hasPermission = true;
    notifyListeners();
  }

  bool? getHasPermission() {
    return _hasPermission;
  }

  void setLocationInfo(
      String? city, String? address, String latitude, String longitude) {
    _city = city;
    _address = address;
    _latitude = latitude;
    _longitude = longitude;
    notifyListeners();
  }

  String getAddres() {
    return _address??'';
  }

  void setName(String name) {
    this._address = name;
  }

  String? getCityId() {
    return _cityId;
  }

  String? getCity() {
    return _city;
  }

  String getLatitude() {
    return _latitude;
  }

  String getLongitude() {
    return _longitude;
  }

  bool isClickCitySelect() {
    return _isClickCitySelect;
  }

  void setIsClickCitySelect(bool isc) {
    _isClickCitySelect = isc;
  }
}
