import 'package:flutter/material.dart';
import 'package:common_module/base/base_umeng_page_view_item_state.dart';
import './components/title_bar.dart';
import './components/search_area.dart';
import 'package:common_module/utils/xapp_utils.dart';
import './components/scroll_body.dart';
import 'package:mvp/providers/provider.dart';
import './actions/action.dart';
import './components/float_banner.dart';
import 'package:common_module/widget/scroll_top_fixed_widget/widget.dart';

class LifeWidget extends StatefulWidget {
  final String isAuditMode;
  LifeWidget({Key? key,required this.isAuditMode}) : super(key: key);
  @override
  State<StatefulWidget> createState() => _LifeState();
}

class _LifeState extends BaseUmengPageViewItemState<LifeWidget>
    with TickerProviderStateMixin {
  ValueNotifier<double> scrollOffset = ValueNotifier<double>(0.0);

  IAction iAction = IAction();
  final ValueNotifier<bool> floatBannerNotifier = ValueNotifier(true);
  ScrollToModel scrollToModel = ScrollToModel();

  @override
  void initState() {
    super.initState();
    scrollToModel.addScrollListener();
  }

  @override
  Widget build(BuildContext context) {
    super.build(context);
    return Scaffold(
      backgroundColor: rgba(245, 245, 245, 1),
      floatingActionButton: ScrollToTopWidget(
        model: scrollToModel,
      ),
      body: SafeArea(
        bottom: false,
        child: ActionProvider(
          action: iAction,
          builder: (context, child) {
            return Stack(
              children: [
                Positioned(
                  top: 44.rpx,
                  left: 0,
                  right: 0,
                  bottom: 0,
                  child: NotificationListener<ScrollNotification>(
                      onNotification: (ScrollNotification notification) {
                        switch (notification.runtimeType) {
                          case ScrollStartNotification:
                            floatBannerNotifier.value = false;
                            break;

                          case ScrollEndNotification:
                            floatBannerNotifier.value = true;
                            break;
                        }
                        return true;
                      },
                      child: ScrollBody(
                        isAuditMode: widget.isAuditMode,
                        iAction: iAction,
                        controller: scrollToModel.scrollController,
                        onScroll: (offset) {
                          scrollOffset.value = offset;
                        },
                      )),
                ),
                Positioned(
                  top: 0,
                  left: 0,
                  right: 0,
                  child: TitleBar(),
                ),
                Positioned(
                  top: 0,
                  left: 0,
                  right: 0,
                  child: SearchArea(
                    scrollOffset: scrollOffset,
                    iAction: iAction,
                  ),
                ),
                
                Positioned(
                  right: 10.rpx,
                  bottom: 100.rpx,
                  child: ValueListenableBuilder<bool>(
                      valueListenable: floatBannerNotifier,
                      builder: (BuildContext context, bool isScrollTStop,
                          Widget? child) {
                        return AnimatedContainer(
                          duration: Duration(milliseconds: 100),
                          transform: Matrix4.translationValues(
                              isScrollTStop ? 0 : 60.rpx, 0, 0),
                          child: FloatBanner(
                            model: iAction.getMiniBannerModel(),
                          ),
                        );
                      }),
                ),
              ],
            );
          },
        ),
      ),
    );
  }

  @override
  void onResume() async {
    super.onResume();
    if (!iAction.getModel()!.getHasPermission()! &&
        !iAction.getModel()!.isClickCitySelect()) {
      await iAction.getModel()!.findPermission();
      if (iAction.getModel()!.getHasPermission()!) {
        iAction.initLocation();
      }
    }
    if (iAction.getModel()!.isClickCitySelect()) {
      iAction.getModel()!.setIsClickCitySelect(false);
    }
  }

  @override
  String getPageViewName() {
    return 'entry_life_widget';
  }

  @override
  Map<String, String> getPageViewArgs() {
    return {};
  }
}
