import 'package:mvp/mvp.dart';
import '../models/model.dart';
import 'package:life_module/models/filter_model.dart';
import 'package:life_module/models/shop_list_model.dart';
import 'package:common_module/utils/loading_dialog_utils.dart';

class IAction extends BaseAction<Model> {
  String? _cityId = '';
  String? _latitude = '';
  String? _longitude = '';
  String? _cityName = '';

  FilterModel _filterModel = FilterModel();
  ShopListModel _shopListModel = ShopListModel();

  IAction(this._cityId, this._latitude, this._longitude, this._cityName);

  @override
  void initState() {
    _init();
  }

  void _init() async {
    setModel(Model(_cityId, _latitude, _longitude, _cityName));
    if (_cityId != null && _cityId!.isNotEmpty) {
      await getFilterModel().getFilterList(_cityId!);
    }
  }

  Future<void> refreshShopList() async {
    var loading = LoadingDialogUtils.getInstance()!.show();
     Future.delayed(Duration(seconds: 2)).then((value) {
      if (loading != null) {
        loading.close();
      }
    });
    try {
      getShopListModel()
          .setShopTypeNavIndex(getFilterModel().getShopTypeNavIndex());
      getShopListModel().setCityId(getModel()!.getCityId());
      getShopListModel().setLat(getModel()!.getLatitude());
      getShopListModel().setLng(getModel()!.getLongitude());
      getShopListModel()
          .setFirstCateId(getFilterModel().getCheckedOneCategory()!.id);
      getShopListModel()
          .setSecondCateId(getFilterModel().getCheckedTwoCategory()!.id ?? "");
      if (getFilterModel().getCheckedTwoRegion()!.id == null) {
        getShopListModel()
            .setRegionId(getFilterModel().getCheckedOneRegion()!.id);
      } else {
        getShopListModel()
            .setRegionId(getFilterModel().getCheckedTwoRegion()!.id);
      }
      getShopListModel().setSortType(getFilterModel().getCheckedOneSort()!.id);
      getShopListModel().clear();
      await getShopListModel().refresh();
      getModel()!.notifyListeners();
    } catch (e) {}
     if (loading != null) {
        loading.close();
      }
  }

  void refresh() async {
    String? cityId = getModel()!.getCityId();

    if (cityId != null &&
        cityId.isNotEmpty &&
        getFilterModel().getCheckedOneCategory() != null) {
      await refreshShopList();
    }
  }

  FilterModel getFilterModel() {
    return _filterModel;
  }

  ShopListModel getShopListModel() {
    return _shopListModel;
  }
}
