import 'package:flutter/material.dart';
import 'package:life_module/components/persistent_header_delegate.dart';
import 'package:life_module/components/shop_type_nav.dart';
import 'package:life_module/components/filter_area.dart';
import 'package:life_module/components/shop_class_nav.dart';
import 'package:life_module/components/shop_list.dart';
import '../actions/action.dart';
import 'package:flutter/rendering.dart';
import 'package:common_module/widget/pull_widget/widget.dart';
import 'package:mvp/providers/provider.dart';
import 'package:common_module/utils/list_model_status_utils.dart';
import 'package:common_module/utils/navigate_utils.dart';
import 'package:life_module/route/index.dart';
import 'package:common_module/utils/xapp_utils.dart';

class ScrollBody extends StatefulWidget {
  final IAction? iAction;
  ScrollBody(this.iAction);
  @override
  State<StatefulWidget> createState() => _ScrollBodyState();
}

class _ScrollBodyState extends State<ScrollBody> {
  IAction? get iAction => widget.iAction;

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      child: QMProvider.value(
        model: iAction!.getShopListModel(),
        builderWidget: (context, dynamic model, child) {
          return PullWidget(
            headerInsertIndex: 3,
            controller:iAction!.getShopListModel().getRefreshController(),
            onLoad: () {
              iAction!.getShopListModel().getData();
            },
            onRefresh: () {
              iAction!.getShopListModel().refresh();
            },
            child: CustomScrollView(
              slivers: [
                SliverPersistentHeader(
                  pinned: false,
                  delegate: PersistentHeaderDelegate(
                    minHeight: 37.rpx,
                    maxHeight: 37.rpx,
                    child: ShopTypeNav(
                      model: iAction!.getFilterModel(),
                      onChange: () {
                        iAction!.refreshShopList();
                      },
                    ),
                  ),
                ),
          
          SliverPersistentHeader(
              pinned: true,
              delegate: PersistentHeaderDelegate(
                  minHeight: 48.rpx,
                  maxHeight: 48.rpx,
                  child: FilterArea(
                    topOffset: 81.rpx - 37.rpx,
                    model: widget.iAction!.getFilterModel(),
                    onTap: (c) {
                      var _context =
                          widget.iAction!.getShopListModel().getContext()!;
                      RenderSliver? renderSliver = _context
                          .findAncestorRenderObjectOfType<RenderSliver>();
                      if (renderSliver != null &&
                          renderSliver.constraints.overlap <= 0) {
                        Scrollable.ensureVisible(_context);
                      }
                    },
                    onChange: () {
                      widget.iAction!.refreshShopList();
                    },
                  ))),
          SliverPersistentHeader(
              pinned: true,
              delegate: PersistentHeaderDelegate(
                  minHeight: 40.rpx,
                  maxHeight: 40.rpx,
                  child: ShopClassNav(
                    model: widget.iAction!.getFilterModel(),
                    onChange: () {
                      widget.iAction!.refreshShopList();
                    },
                  ))),
          

                          Builder(
                            builder: (context) {
                              Widget? statusWidget =
                                  ListModelStatusUtils.getInstance()!.form(
                                      model: model,
                                      onTap: () {
                                        iAction!.refresh();
                                      });
                              if (statusWidget != null) {
                                return SliverToBoxAdapter(
                                  child: statusWidget,
                                );
                              }
                              return ShopList(
                                model: iAction!.getShopListModel(),
                                onReset: () {
                                  widget.iAction!.getFilterModel().reset();
                                  widget.iAction!.refreshShopList();
                                },
                                onTap: (item) {
                                  NavigateUtils.push(
                                    path: LIFE_SHOP_PAGE,
                                    arguments: {
                                      'shopId': item.combo != null
                                          ? item.combo!.shopId
                                          : item.restaurant!.shopId,
                                      'lat': widget.iAction!
                                          .getModel()!
                                          .getLatitude(),
                                      'lng': widget.iAction!
                                          .getModel()!
                                          .getLongitude(),
                                      'firstCateId': widget.iAction!
                                          .getFilterModel()
                                          .getCheckedOneCategory()!
                                          .id,
                                      'itemId': item.combo != null
                                          ? item.combo!.dealId
                                          : "",
                                      "cityName": widget.iAction!
                                          .getModel()!
                                          .getCityName()
                                    },
                                    isNative: false,
                                  );
                                },
                              );
                            },
                          )
                        ],
                      ),
                    );
                  }),
            )
          ;
  }
}
