import 'package:flutter/material.dart';
import 'package:common_module/utils/xapp_utils.dart';
import 'package:common_module/components/i_app_bar.dart';
import 'package:life_module/components/search_bar.dart';
import 'package:life_module/utils/filter_panel_utils.dart';
import 'package:life_module/views/search/models/model.dart';
import './actions/action.dart';
import 'package:flutter/rendering.dart';
import 'package:mvp/providers/provider.dart';
import './components/scroll_body.dart';
import './components/search_tips.dart';
import './components/search_history.dart';
import '../../util/search_history_utils.dart';

class LifeSearchPage extends StatefulWidget {
  final Map<String, dynamic>? param;
  LifeSearchPage({this.param});
  @override
  State<StatefulWidget> createState() => _SearchPageState();
}

class _SearchPageState extends State<LifeSearchPage> with TickerProviderStateMixin {
  IAction? iAction;
  FocusNode focusNode = FocusNode();
  TextEditingController controller = TextEditingController();
  @override
  void initState() {
    super.initState();
    iAction = IAction(widget.param!['cityId'], widget.param!['latitude'],widget.param!['longitude'],widget.param!['cityName']);
    Future.delayed(Duration(milliseconds: 300)).then((value) {
      focusNode.requestFocus();
    });
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(child:Scaffold(
      appBar: IAppBar(
        leading: true,
        elevation: 0,
        color: Colors.white,
        child: Container(
          padding: EdgeInsets.only(left: 42.rpx, right: 13.rpx),
          child: SearchBar(
            enabled: true,
            focusNode: focusNode,
            controller: controller,
            onChanged: (s) {
              if (iAction!.getFilterModel().getCheckedOneCategory() != null) {
                iAction!.getModel()!.findSearchTips(
                    iAction!.getFilterModel().getCheckedOneCategory()!.id, s);
              }
              if (s.isEmpty) {
                iAction!.getShopListModel().setKeywords("");
                iAction!.getShopListModel().clear();
              }
            },
            onSubmitted: (s) {
              search(s);
            },
          ),
        ),
      ),
      body: ActionProvider<Model,IAction>(
        action: iAction!,
        builder: (context, model) {
          if (iAction!.getModel()!.getSearchTipsList().length > 0) {
            return SearchTips(
              items: iAction!.getModel()!.getSearchTipsList(),
              keywords: iAction!.getModel()!.getKeywords(),
              onTap: (item) {
                controller.text = item.name!;
                iAction!.getModel()!.clearSearchTipsList();
                search(item.name!);
              },
            );
          } else if (iAction!.getShopListModel().getKeywords().isNotEmpty) {
            return ScrollBody(iAction);
          }
          return SearchHistory(
            onTap: (s) {
              search(s);
            },
          );
        },
      ),
    ),onWillPop:(){
      if(FilterPanelUtils.overlayEntry != null){
        FilterPanelUtils.close();
        return Future.value(false);
      }
      return Future.value(true);
    },);
  }

  void search(String wrods) {
    controller.text = wrods;
    iAction!.getModel()!.clearSearchTipsList();
    SearchHistoryUtils.getInstance()!.push(wrods);
    focusNode.unfocus();
    iAction!.getShopListModel().setKeywords(wrods);
    if (wrods.isNotEmpty) {
      iAction!.refresh();
    } else {
      iAction!.getShopListModel().clear();
    }
  }
}
