import 'package:flutter/material.dart';
import 'package:common_module/utils/xapp_utils.dart';
import '../models/banner_model.dart';
import 'package:common_module/widget/xiaoxiong_base_image_widget/widget.dart';
import 'package:mvp/mvp.dart';
import 'package:life_module/utils/activity_config_tap_utils.dart';
import 'package:flutter_swiper/flutter_swiper.dart';

class AdBanner extends StatelessWidget {
  final BannerModel? model;
  AdBanner(this.model);

  @override
  Widget build(BuildContext context) {
    ValueNotifier<int?> scrollIndex = ValueNotifier<int?>(0);
    return SliverToBoxAdapter(
      child: QMProvider<BannerModel>.value(
        model: model!,
        builderWidget: (context, model, child) {
          if (model.getList().length == 0) {
            return SizedBox();
          }
          return Container(
              padding: EdgeInsets.symmetric(horizontal: 13.rpx),
              height: 90.rpx,
              child: Stack(
                children: [
                  ClipRRect(
                    borderRadius: BorderRadius.circular(8.rpx),
                    child: Swiper(
                      key: UniqueKey(),
                      itemCount: model.getList().length,
                      autoplay: model.getList().length <= 1,
                      loop: model.getList().isNotEmpty &&
                          model.getList().length > 1,
                      onTap: (index) {
                        var item = model.getList()[index];
                        AppTapConfigUtils.getInstance()!.activityConfigTap(item.toJson());
                      },
                      onIndexChanged: (index) {
                        scrollIndex.value = index;
                      },
                      itemBuilder: (context, index) {
                        var item = model.getList()[index];
                        return XiaoxiongBaseImageWidget(
                          imageUrl: item.imgUrl!,
                          height: 90.rpx,
                          fit: BoxFit.cover,
                        );
                      },
                    ),
                  ),
                  Positioned(
                    bottom: 5.rpx,
                    left: 0,
                    right: 0,
                    child: _SecKillPagination(
                      scrollIndex: scrollIndex,
                      length: model.getList().length,
                    ),
                  )
                ],
              ));
        },
      ),
    );
  }
}

/// 指示器
class _SecKillPagination extends StatelessWidget {
  final int? length;
  final ValueNotifier<int?>? scrollIndex;
  _SecKillPagination({this.scrollIndex, this.length});
  @override
  Widget build(BuildContext context) {
    if (length == 0) {
      return SizedBox(
        height: 4.rpx,
      );
    }
    return ValueListenableBuilder<int?>(
        valueListenable: scrollIndex!,
        builder: (BuildContext context, dynamic offset, Widget? child) {
          List<Widget> items = List.generate(length!.ceil(), (index) {
            Color color = rgba(255, 255, 255, 0.5);
            if (index == scrollIndex!.value) {
              color = rgba(255, 255, 255, 1);
            }
            return Container(
              width: 4.rpx,
              height: 4.rpx,
              margin: EdgeInsets.symmetric(horizontal: 2.rpx),
              decoration: BoxDecoration(
                  color: color, borderRadius: BorderRadius.circular(2.rpx)),
            );
          });
          return Container(
            height: 4.rpx,
            alignment: Alignment.center,
            child: Row(
              mainAxisAlignment: MainAxisAlignment.center,
              children: items,
            ),
          );
        });
  }
}
