import 'package:flutter/material.dart';
import 'package:common_module/utils/xapp_utils.dart';
import 'package:life_module/components/life_image.dart';
import '../models/sec_kill_model.dart';
import 'package:mvp/mvp.dart';
import 'package:life_repository/life_repository.dart';
import 'package:common_module/widget/xiaoxiong_base_image_widget/widget.dart';
import 'package:common_module/utils/meituan_utils.dart';
import 'package:common_module/utils/list_model_status_utils.dart';
import '../actions/action.dart';
import '../../../utils/buy_loading_dialog_utils.dart';
import 'package:common_module/utils/image_preview_utils.dart';

class GoodsItems extends StatelessWidget {
  final IAction? iAction;
  GoodsItems(this.iAction);
  @override
  Widget build(BuildContext context) {
    return SliverPadding(
        padding: EdgeInsets.zero,
        sliver: QMProvider<SecKillModel>.value(
          model: iAction!.getSecKillModel()!,
          builderWidget: (context, model, child) {
            Widget? statusWidget = ListModelStatusUtils.getInstance()!.form(
                model: model,
                onTap: () {
                  model.refresh();
                });
            if (statusWidget != null) {
              return SliverToBoxAdapter(
                child: statusWidget,
              );
            }
            return SliverList(
                delegate: SliverChildBuilderDelegate(
              (BuildContext context, int index) {
                return _GoodsItem(
                  enabled: index == 1,
                  item: model.getList()[index],
                  iAction: iAction,
                );
              },
              childCount: model.getList().length,
            ));
          },
        ));
  }
}

class _GoodsItem extends StatelessWidget {
  final SecKillGoods? item;
  final bool enabled;
  final IAction? iAction;
  _GoodsItem({this.enabled = false, this.item, this.iAction});
  @override
  Widget build(BuildContext context) {
    return Container(
      margin: EdgeInsets.only(left: 13.rpx, right: 13.rpx, bottom: 8.rpx),
      padding: EdgeInsets.only(
        left: 12.rpx,
        right: 12.rpx,
        top: 12.rpx,
      ),
      decoration: BoxDecoration(
          color: Colors.white, borderRadius: BorderRadius.circular(8.rpx)),
      child: Column(
        children: [
          Container(
            height: 102.rpx,
            child: Row(
              children: [
                InkWell(
                    onTap: () {
                      ImagePreviewUtils.showIndex(
                          context: context,
                          images: [item!.itemPic!],
                          heroTag: item!.itemPic);
                    },
                    child: ClipRRect(
                        borderRadius: BorderRadius.circular(8.rpx),
                        child: Container(
                          height: 90.rpx,
                          width: 90.rpx,
                          child: XiaoxiongBaseImageWidget(
                            imageUrl: item!.itemPic!,
                            height: 90.rpx,
                            width: 90.rpx,
                            fit: BoxFit.cover,
                          ),
                        ))),
                SizedBox(
                  width: 8.rpx,
                ),
                Expanded(
                  child: Stack(
                    children: [
                      Container(
                        height: 90.rpx,
                        child: Column(
                          crossAxisAlignment: CrossAxisAlignment.start,
                          children: [
                            _Title(item),
                            SizedBox(
                              height: 6.rpx,
                            ),
                            _PriceInfo(item),
                            SizedBox(
                              height: 6.rpx,
                            ),
                            _Commission(item),
                            SizedBox(
                              height: 6.rpx,
                            ),
                            _Progress(item),
                          ],
                        ),
                      ),
                      _BuyBtn(item),
                    ],
                  ),
                ),
              ],
            ),
          ),
          _LocaionInfo(item, iAction),
        ],
      ),
    );
  }
}

/// 标题
class _Title extends StatelessWidget {
  final SecKillGoods? item;
  _Title(this.item);
  @override
  Widget build(BuildContext context) {
    return Container(
      height: 21.rpx,
      child: Row(
        children: [
          Expanded(
            child: Container(
              child: Text(
                item!.itemTitle!,
                maxLines: 1,
                overflow: TextOverflow.ellipsis,
                style: TextStyle(
                    color: rgba(48, 38, 0, 1),
                    fontSize: 15.rpx,
                    height: 1.rpx,
                    fontWeight: FontWeight.bold),
              ),
            ),
          )
        ],
      ),
    );
  }
}

class _Commission extends StatelessWidget {
  final SecKillGoods? item;
  _Commission(this.item);
  @override
  Widget build(BuildContext context) {
    return Container(
      height: 18.rpx,
      padding: EdgeInsets.symmetric(horizontal: 4.rpx, vertical: 2.rpx),
      decoration: BoxDecoration(
          border: Border.all(color: rgba(255, 128, 0, 1), width: 0.5),
          borderRadius: BorderRadius.only(
              topLeft: Radius.circular(9.rpx),
              topRight: Radius.circular(9.rpx),
              bottomRight: Radius.circular(9.rpx))),
      child: Text(
        item!.userCommissionText!,
        style: TextStyle(
            color: rgba(255, 128, 0, 1), fontSize: 11.rpx, height: 1.2),
      ),
    );
  }
}

/// 价格信息
class _PriceInfo extends StatelessWidget {
  final SecKillGoods? item;
  _PriceInfo(this.item);
  @override
  Widget build(BuildContext context) {
    return Container(
      height: 21.rpx,
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.baseline,
        children: [
          Text(
            "¥",
            style: TextStyle(
              color: rgba(255, 4, 0, 1),
              fontSize: 11.rpx,
            ),
          ),
          Text(
            item!.endPrice!,
            style: TextStyle(
                color: rgba(255, 4, 0, 1), fontSize: 18.rpx, height: 1.2.rpx),
          ),
          SizedBox(
            width: 4.rpx,
          ),
          Container(
            child: Text(
              "秒杀价",
              style: TextStyle(
                  color: rgba(255, 4, 0, 1), fontSize: 11.rpx, height: 1.rpx),
            ),
          ),
          SizedBox(
            width: 4.rpx,
          ),
          Text(
            "¥${item!.originalPrice}",
            style: TextStyle(
                color: rgba(153, 153, 153, 1),
                fontSize: 11.rpx,
                decoration: TextDecoration.lineThrough),
          ),
        ],
      ),
    );
  }
}

/// 马上抢
class _BuyBtn extends StatelessWidget {
  final SecKillGoods? item;
  _BuyBtn(this.item);
  @override
  Widget build(BuildContext context) {
    bool enabled = item!.isRedirect == '2';
    return Positioned(
      right: 0,
      bottom: 0,
      width: 54.rpx,
      height: 32.rpx,
      child: InkWell(
        onTap: () {
          if (enabled) {
            XAppUtils.getInstance()!.toast(item!.menuText!);
            return;
          }
          var loading = BuyLoadingDialogUtils.getInstance()!.show();
          Future.delayed(Duration(seconds: 1)).then((value) {
            loading.close();
            MeituanUtils.getInstance()!.openByUrl(item!.couponUrl!);
          });
        },
        child: Container(
          alignment: Alignment.center,
          decoration: BoxDecoration(
              color: rgba(216, 216, 216, 1),
              gradient: enabled
                  ? null
                  : LinearGradient(
                      begin: Alignment.topCenter,
                      end: Alignment.bottomCenter,
                      colors: [rgba(255, 128, 0, 1), rgba(255, 4, 0, 1)]),
              borderRadius: BorderRadius.circular(20.rpx)),
          child: Text(
            item!.menuText!,
            style: TextStyle(
                color: rgba(255, 255, 255, 1),
                fontSize: 13.rpx,
                height: 1.2.rpx),
          ),
        ),
      ),
    );
  }
}

/// 进度
class _Progress extends StatelessWidget {
  final SecKillGoods? item;
  _Progress(this.item);
  @override
  Widget build(BuildContext context) {
    double maxWidth = 154.rpx;
    double whiteTextWidth = 45.rpx;
    double soldPercent = double.parse(item!.soldPercent!.replaceAll('%', ''));
    double p = soldPercent / 100 * maxWidth;
    bool showWhiteText = p >= whiteTextWidth;
    return Container(
      height: 12.rpx,
      width: maxWidth,
      child: Stack(
        children: [
          Container(
            alignment: Alignment.centerLeft,
            padding: EdgeInsets.only(left: p + 7.rpx),
            decoration: BoxDecoration(
                color: rgba(245, 245, 245, 1),
                borderRadius: BorderRadius.circular(20.rpx)),
            child: showWhiteText
                ? SizedBox()
                : Text(
                    "已抢${item!.soldPercent}",
                    style: TextStyle(
                        color: rgba(153, 153, 153, 1),
                        fontSize: 9.rpx,
                        height: 1.rpx),
                  ),
          ),
          Container(
            height: 12.rpx,
            width: p,
            alignment: Alignment.centerLeft,
            padding: EdgeInsets.only(left: 6.rpx),
            decoration: BoxDecoration(
                gradient: LinearGradient(
                    begin: Alignment.topCenter,
                    end: Alignment.bottomCenter,
                    colors: [rgba(255, 128, 0, 1), rgba(255, 4, 0, 1)]),
                borderRadius: BorderRadius.circular(20.rpx)),
            child: !showWhiteText
                ? SizedBox()
                : Text(
                    "已抢${item!.soldPercent}",
                    maxLines: 1,
                    style: TextStyle(
                        color: Colors.white, fontSize: 9.rpx, height: 1.rpx),
                  ),
          )
        ],
      ),
    );
  }
}

/// 地址信息
class _LocaionInfo extends StatelessWidget {
  final SecKillGoods? item;
  final IAction? iAction;
  _LocaionInfo(this.item, this.iAction);
  @override
  Widget build(BuildContext context) {
    return InkWell(
        onTap: () async {
          //String gps = await AmapUtils.getInstance() .addressToGps(item.address, iAction.getCityName());
          //MapUtils.getInstance().open(gps, item.address);
        },
        child: Container(
          height: 34.rpx,
          padding: EdgeInsets.only(bottom: 4.rpx),
          child: Row(
            children: [
              LifeImage(
                name: "assets/images/sec_kill/ic_gps_gray.png",
                width: 20.rpx,
                height: 20.rpx,
              ),
              Container(
                constraints: BoxConstraints(maxWidth: 260.rpx),
                child: Text(
                  item!.address!,
                  maxLines: 1,
                  overflow: TextOverflow.ellipsis,
                  style: TextStyle(
                      color: rgba(102, 102, 102, 1),
                      fontSize: 11.rpx,
                      height: 1.rpx),
                ),
              ),
              /*LifeImage(
                name: "assets/images/life/ic_miaosha_arrow_right.png",
                width: 8.rpx,
                height: 8.rpx,
              ),*/
              Spacer(),
              Text(
                item!.distance!,
                style: TextStyle(
                    color: rgba(153, 153, 153, 1),
                    fontSize: 12.rpx,
                    height: 1.rpx),
              )
            ],
          ),
        ));
  }
}
