import 'package:flutter/material.dart';
import 'package:common_module/utils/xapp_utils.dart';
import 'package:life_module/components/life_image.dart';
import 'package:life_module/util/resource.dart';
import '../models/model.dart';
import 'package:common_module/utils/loading_dialog_utils.dart';
import 'package:common_module/base/base_route_widget_state.dart';

class LocaltionTips extends StatefulWidget {
  final Model? model;
  final Function(String? cityId, String lat, String lon)? onLocationChange;
  LocaltionTips({this.model, this.onLocationChange});

  @override
  State<StatefulWidget> createState() => LocaltionTipsState();
}

class LocaltionTipsState extends BaseRouteWeigetState<LocaltionTips> {
  Model? get model => widget.model;
  @override
  Widget build(BuildContext context) {
    if (model!.hasPermission()!) {
      return SliverToBoxAdapter(
        child: SizedBox(),
      );
    }
    return SliverToBoxAdapter(
      child: Container(
        height: 56.rpx,
        margin: EdgeInsets.only(left: 13.rpx, top: 8.rpx, right: 13.rpx),
        decoration: BoxDecoration(
            color: Colors.white, borderRadius: BorderRadius.circular(8.rpx)),
        child: Row(
          children: [
            SizedBox(
              width: 12.rpx,
            ),
            Expanded(
              child: Container(
                height: 36.rpx,
                child: Text(
                  "未获取定位权限，以下商品门店距您所在位置可能会存在偏差，建议您开启定位获取精准门店商品",
                  style: TextStyle(
                      color: rgba(48, 38, 0, 1),
                      fontSize: 11.rpx,
                      height: 1.6.rpx),
                ),
              ),
            ),
            SizedBox(
              width: 10.rpx,
            ),
            InkWell(
                onTap: () async {
                  await model!.reqPermission();
                  hasPermission();
                },
                child: Container(
                  width: 64.rpx,
                  height: 26.rpx,
                  alignment: Alignment.center,
                  decoration: BoxDecoration(
                      color: rgba(255, 128, 0, 0.1),
                      borderRadius: BorderRadius.circular(8.rpx)),
                  child: Text(
                    "开启定位",
                    style: TextStyle(
                        color: rgba(255, 128, 0, 1),
                        fontSize: 12.rpx,
                        height: 1.rpx),
                  ),
                )),
            InkWell(
                onTap: () {
                  model!.setPermission();
                },
                child: Container(
                  width: 17.rpx,
                  height: 56.rpx,
                  alignment: Alignment.topRight,
                  child: LifeImage(
                    name: "assets/images/sec_kill/ic_cycle_close.png",
                    width: 17.rpx,
                    height: 17.rpx,
                  ),
                ))
          ],
        ),
      ),
    );
  }

  void hasPermission() async {
    if (model!.hasPermission()!) {
      var loading = LoadingDialogUtils.getInstance()!.show();
      try {
        var location = await model!.getLocation();
        if((location?.address??'').isEmpty){
          String myLocation = '${location.longitude},${location.latitude}';
          location = await Res.convertLocation(entity:location,location:myLocation);
        }
        var cityId = await model!.findCityId(location.city!);
        loading.close();
        widget.onLocationChange!(cityId, location.latitude.toString(),
            location.longitude.toString());
      } catch (e) {
        loading.close();
      }
    }
  }

  @override
  void onResume() async {
    super.onResume();
    if (!model!.hasPermission()!) {
      await model!.findPermission();
      hasPermission();
    }
  }
}
