import 'dart:async';

import 'package:mvp/mvp.dart';
import 'package:common_module/utils/amap_utils.dart';
import 'package:modal/modal.dart';
import 'package:life_repository/life_repository.dart';

class Model extends BaseModel {
  bool? _hasPermission = true;
  List<SeckillSession> _seckillSessionList = [];

  Future<void> findPermission() async {
    _hasPermission = await AmapUtils.getInstance()!.hasPermission();
    if (_hasPermission!) {
      _hasPermission = await AmapUtils.getInstance()!.isOpenGPS;
    }
    notifyListeners();
  }

  bool? hasPermission() {
    return _hasPermission;
  }

  void setPermission() {
    _hasPermission = true;
    notifyListeners();
  }

  List<SeckillSession> getSeckillSessionList() {
    return _seckillSessionList;
  }

  Future<LocationEntity> getLocation() {
    return AmapUtils.getInstance()!.getLocation();
  }

  Future<String?> findCityId(String city) async {
    return CityRepository.get().getCityId(city);
  }

  Future<void> findSeckillSession(String cityId) async {
    _seckillSessionList =
        await SecKillRepository.get().findSeckillSession(cityId, '2');
    notifyListeners();
  }

  Future<void> reqPermission() async {
    bool confirm = await (Modal.showModal(
        title: "温馨提示",
        msg: "为给您提供更好的服务，需获取您的位置权限",
        cancelBtnText: '拒绝',
        confirmBtnText: '同意') as FutureOr<bool>);
    if (!confirm) {
      return;
    }
    _hasPermission = await AmapUtils.getInstance()!.reqPermission();

    if (_hasPermission!) {
      _hasPermission = await AmapUtils.getInstance()!.isOpenGPS;
      if (!_hasPermission!) {
        confirm = await (Modal.showModal(
            title: "温馨提示",
            msg: "为给您提供更好的服务，请打开GPS",
            cancelBtnText: '拒绝',
            confirmBtnText: '同意') as FutureOr<bool>);
        if (!confirm) {
          return;
        }
        _hasPermission = await AmapUtils.getInstance()!.enableGPS();
      }
    }
    notifyListeners();
  }
}
