import 'package:flutter/material.dart';
import 'package:common_module/utils/xapp_utils.dart';
import 'package:life_module/views/shop/models/model.dart';
import 'package:life_module/components/type_icon.dart';
import 'package:life_repository/life_repository.dart';
import 'package:common_module/widget/xiaoxiong_base_image_widget/widget.dart';
import 'package:common_module/utils/meituan_utils.dart';
import '../../../utils/buy_loading_dialog_utils.dart';
import 'package:common_module/utils/image_preview_utils.dart';

class GoodsItems extends StatelessWidget {
  final Model? model;
  GoodsItems(this.model);
  @override
  Widget build(BuildContext context) {
    return SliverList(
        delegate: SliverChildBuilderDelegate(
      (BuildContext context, int index) {
        return _GoodsItem(
          isFirst: index == 0,
          isLast: index == 1,
          item: model!.getShopInfoEntity()!.shopCouponItems![index],
        );
      },
      childCount: model!.getShopInfoEntity()!.shopCouponItems!.length,
    ));
  }
}

class _GoodsItem extends StatelessWidget {
  final bool? isFirst;
  final bool? isLast;
  final ShopCouponItem? item;
  _GoodsItem({this.isFirst, this.isLast, this.item});
  @override
  Widget build(BuildContext context) {
    double top = isFirst! ? 16.rpx : 12.rpx;
    return Container(

      height: 90.rpx + top,
      padding: EdgeInsets.only(
          left: 12.rpx, right: 12.rpx, top: top, bottom: 16.rpx),
      decoration: BoxDecoration(
        color: Colors.white,
          borderRadius: BorderRadius.vertical(bottom: Radius.circular(isLast! ? 8.rpx : 0))),
      child: Row(
        children: [
          InkWell(
            child: ClipRRect(
                borderRadius: BorderRadius.circular(8.rpx),
                child: Container(
                  height: 74.rpx,
                  width: 74.rpx,
                  child: XiaoxiongBaseImageWidget(
                    imageUrl: item!.itemImg!,
                    height: 74.rpx,
                    width: 74.rpx,
                    fit: BoxFit.cover,
                  ),
                )),
            onTap: () {
              ImagePreviewUtils.showIndex(
                  context: context,
                  images: [item!.itemImg!],
                  heroTag: item!.itemImg);
            },
          ),
          SizedBox(
            width: 12.rpx,
          ),
          Expanded(
            child: Stack(
              children: [
                Container(
                  height: 74.rpx,
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      _Title(item),
                      SizedBox(
                        height: 6.rpx,
                      ),
                      _Commission(item),
                      SizedBox(
                        height: 8.rpx,
                      ),
                      _PriceInfo(item),
                    ],
                  ),
                ),
                _BuyBtn(item),
              ],
            ),
          ),
        ],
      ),
    );
  }
}

/// 标题
class _Title extends StatelessWidget {
  final ShopCouponItem? item;
  _Title(this.item);
  @override
  Widget build(BuildContext context) {
    return Container(
      height: 21.rpx,
      child: Row(
        children: [
          TypeIcon(
            type: int.parse(item!.dealType!),
          ),
          SizedBox(
            width: 4.rpx,
          ),
          Expanded(
            child: Container(
              child: Text(
                item!.itemTitle!,
                maxLines: 1,
                overflow: TextOverflow.ellipsis,
                style: TextStyle(
                    color: rgba(48, 38, 0, 1),
                    fontSize: 15.rpx,
                    height: 1.rpx,
                    fontWeight: FontWeight.bold),
              ),
            ),
          )
        ],
      ),
    );
  }
}

class _Commission extends StatelessWidget {
  final ShopCouponItem? item;
  _Commission(this.item);
  @override
  Widget build(BuildContext context) {
    if (double.parse(item!.userCommission!) == 0) {
      Container(height: 18.rpx);
    }
    return Container(
      height: 18.rpx,
      padding: EdgeInsets.symmetric(horizontal: 4.rpx, vertical: 2.rpx),
      decoration: BoxDecoration(
          border: Border.all(color: rgba(255, 128, 0, 1), width: 0.5),
          borderRadius: BorderRadius.only(
              topLeft: Radius.circular(9.rpx),
              topRight: Radius.circular(9.rpx),
              bottomRight: Radius.circular(9.rpx))),
      child: Text(
        item!.userCommissionText!,
        style: TextStyle(
            color: rgba(255, 128, 0, 1), fontSize: 11.rpx, height: 1.2),
      ),
    );
  }
}

/// 价格信息
class _PriceInfo extends StatelessWidget {
  final ShopCouponItem? item;
  _PriceInfo(this.item);
  @override
  Widget build(BuildContext context) {
    return Container(
      height: 21.rpx,
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.baseline,
        children: [
          Text(
            "¥",
            style: TextStyle(
              color: rgba(255, 4, 0, 1),
              fontSize: 11.rpx,
            ),
          ),
          Text(
            item!.endPrice!,
            style: TextStyle(
              color: rgba(255, 4, 0, 1),
              fontSize: 16.rpx,
            ),
          ),
          SizedBox(
            width: 4.rpx,
          ),
          Text(
            "¥${item!.originalPrice}",
            style: TextStyle(
                color: rgba(153, 153, 153, 1),
                fontSize: 12.rpx,
                decoration: TextDecoration.lineThrough),
          ),
          SizedBox(
            width: 8.rpx,
          ),
          Container(
            padding: EdgeInsets.symmetric(horizontal: 4.rpx, vertical: 3.rpx),
            decoration: BoxDecoration(
                color: rgba(255, 128, 0, 0.06),
                borderRadius: BorderRadius.circular(34.rpx)),
            child: Text(
              "${item!.discountPercent}折",
              style: TextStyle(
                  color: rgba(255, 128, 0, 1), fontSize: 11.rpx, height: 1.rpx),
            ),
          )
        ],
      ),
    );
  }
}

class _BuyBtn extends StatelessWidget {
  final ShopCouponItem? item;
  _BuyBtn(this.item);
  @override
  Widget build(BuildContext context) {
    return InkWell(
      child: Container(
        alignment: Alignment.centerRight,
        child: Container(
          width: 54.rpx,
          height: 32.rpx,
          alignment: Alignment.center,
          decoration: BoxDecoration(
              gradient: LinearGradient(
                  begin: Alignment.topCenter,
                  end: Alignment.bottomCenter,
                  colors: [rgba(255, 128, 0, 1), rgba(255, 4, 0, 1)]),
              borderRadius: BorderRadius.circular(20.rpx)),
          child: Text(
            "抢购",
            style: TextStyle(
                color: rgba(255, 255, 255, 1), fontSize: 13.rpx, height: 1.rpx),
          ),
        ),
      ),
      onTap: () {
        var loading = BuyLoadingDialogUtils.getInstance()!.show();
        Future.delayed(Duration(seconds: 1)).then((value) {
          loading.close();
          MeituanUtils.getInstance()!.openByUrl(item!.couponUrl!);
        });
      },
    );
  }
}
