import 'package:flutter/material.dart';
import 'package:common_module/utils/xapp_utils.dart';
import 'package:life_module/components/life_image.dart';
import 'package:life_module/views/shop/models/model.dart';
import 'package:common_module/widget/xiaoxiong_base_image_widget/widget.dart';
import 'package:flutter_native_toast/flutter_native_toast.dart';
import 'package:common_module/utils/clipboard_utils.dart';
import 'package:common_module/utils/audit_mode_utils.dart';
import 'package:common_module/utils/image_preview_utils.dart';

class ShopInfo extends StatelessWidget {
  final Model? model;
  ShopInfo(this.model);
  @override
  Widget build(BuildContext context) {
    return SliverToBoxAdapter(
      child: ClipRRect(
          borderRadius: BorderRadius.circular(8.rpx),
          child: Container(
            margin: EdgeInsets.only(top: 8.rpx),
            color: Colors.white,
            child: Column(
              children: [
                _ShopInfo(model),
                SizedBox(
                  height: 12.rpx,
                ),
                _Location(model),
                AuditModeUtils.getInstance()!.isAuditMode()
                    ? SizedBox()
                    : Container(
                        height: 48.rpx,
                        child: LifeImage(
                          name: "assets/images/shop/img_step.png",
                          height: 48.rpx,
                        ),
                      )
              ],
            ),
          )),
    );
  }
}

/// 店铺展示信息
class _ShopInfo extends StatelessWidget {
  final Model? model;
  _ShopInfo(this.model);
  @override
  Widget build(BuildContext context) {
    return Container(
      padding: EdgeInsets.symmetric(horizontal: 12.rpx),
      child: Column(
        children: [
          Container(
            alignment: Alignment.centerLeft,
            padding: EdgeInsets.only(top: 12.rpx, bottom: 10.rpx),
            child: Text(
              model!.getShopInfoEntity()!.shopInfo!.shopTitle!,
              style: TextStyle(
                  color: rgba(48, 38, 0, 1), fontSize: 18.rpx, height: 1.rpx,fontWeight:FontWeight.w600),
            ),
          ),
          Container(
            height: 80.rpx,
            child: Row(
              children: [
                InkWell(
                  child: ClipRRect(
                      borderRadius: BorderRadius.circular(8.rpx),
                      child: Container(
                        height: 80.rpx,
                        width: 80.rpx,
                        child: XiaoxiongBaseImageWidget(
                          imageUrl: model!.getShopInfoEntity()!.shopInfo!.shopImg!,
                          height: 80.rpx,
                          width: 80.rpx,
                          fit: BoxFit.cover,
                        ),
                      )),
                  onTap: () {
                    ImagePreviewUtils.showIndex(
                        context: context,
                        images: [model!.getShopInfoEntity()!.shopInfo!.shopImg!],
                        heroTag: model!.getShopInfoEntity()!.shopInfo!.shopImg);
                  },
                ),
                SizedBox(
                  width: 12.rpx,
                ),
                Expanded(
                  child: Container(
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: [
                        _ScoreInfo(model),
                        SizedBox(
                          height: 4.rpx,
                        ),
                        Container(
                          height: 16.rpx,
                          child: Text(
                            "¥${model!.getShopInfoEntity()!.shopInfo!.avgPrice}/人",
                            style: TextStyle(
                                color: rgba(102, 102, 102, 1),
                                fontSize: 11.rpx),
                          ),
                        ),
                        SizedBox(
                          height: 4.rpx,
                        ),
                        // Container(
                        //   height: 16.rpx,
                        //   child: Text(
                        //     "季销${model.getShopInfoEntity().shopInfo.quarterVolume}",
                        //     style: TextStyle(
                        //         color: rgba(102, 102, 102, 1),
                        //         fontSize: 11.rpx),
                        //   ),
                        // ),
                        SizedBox(
                          height: 4.rpx,
                        ),
                        Container(
                          height: 16.rpx,
                          child: Text(
                            "营业时间 ${model!.getShopInfoEntity()!.shopInfo!.businessTime}",
                            style: TextStyle(
                                color: rgba(48, 38, 0, 1), fontSize: 12.rpx),
                          ),
                        )
                      ],
                    ),
                  ),
                )
              ],
            ),
          ),
        ],
      ),
    );
  }
}

/// 评分和距离信息
class _ScoreInfo extends StatelessWidget {
  final Model? model;
  _ScoreInfo(this.model);
  @override
  Widget build(BuildContext context) {
    var showStarScore = model!.getShopInfoEntity()!.shopInfo!.showStarScore!;
    List<InlineSpan> widgets = [];
    double score = double.parse(showStarScore);
    for (int star = 1, len = score.ceil(); star <= len; star++) {
      widgets.add(WidgetSpan(
          child: Image.asset(
        star <= score
            ? "assets/ic_star_small_full.png"
            : "assets/ic_star_small_half.png",
        width: 12.rpx,
        height: 12.rpx,
      )));
    }
    for (int star = widgets.length; star < 5; star++) {
      widgets.add(WidgetSpan(
          child: Image.asset(
        "assets/ic_star_small_blank.png",
        width: 12.rpx,
        height: 12.rpx,
      )));
    }
    widgets.add(WidgetSpan(
        child: SizedBox(
      width: 2.rpx,
    )));
    return Container(
      height: 18.rpx,
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          Container(
            alignment: Alignment.centerLeft,
            height: 14.rpx,
            child: Text.rich(TextSpan(children: widgets)),
          ),
          Container(
            child: Text(
                double.parse(model!.getShopInfoEntity()!.shopInfo!.commentScore!)
                    .toStringAsFixed(1),
                style: TextStyle(
                  color: rgba(255, 128, 0, 1),
                  fontSize: 14.rpx,
                  fontWeight: FontWeight.bold,
                )),
          ),
          Spacer(),
          Text(
            model!.getShopInfoEntity()!.shopInfo!.distance!,
            style: TextStyle(color: rgba(102, 102, 102, 1), fontSize: 11.rpx),
          )
        ],
      ),
    );
  }
}

/// 位置信息
class _Location extends StatelessWidget {
  final Model? model;
  _Location(this.model);
  @override
  Widget build(BuildContext context) {
    return Container(
      padding: EdgeInsets.symmetric(vertical: 8.rpx, horizontal: 12.rpx),
      decoration: BoxDecoration(
          image: DecorationImage(
              alignment: Alignment.topLeft,
              image: AssetImage(
                "assets/images/shop/img_map.png",
                package: "life_module",
              ))),
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          Expanded(
            child: Container(
              child: Text(
                model!.getShopInfoEntity()!.shopInfo!.address!,
                style: TextStyle(color: rgba(48, 38, 0, 1), fontSize: 12.rpx),
              ),
            ),
          ),
          SizedBox(
            width: 12.rpx,
          ),
          InkWell(
            onTap: () {
              ClipboardUtils.copyText(
                  model!.getShopInfoEntity()!.shopInfo!.address!);
              FlutterNativeToast.showToast('门店地址已复制');
            },
            child: Container(
              child: Column(
                children: [
                  Container(
                    height: 20.rpx,
                    width: 20.rpx,
                    child: LifeImage(
                      name: "assets/images/shop/ic_copy.png",
                      width: 20.rpx,
                      height: 20.rpx,
                    ),
                  ),
                  Container(
                    height: 18.rpx,
                    child: Text(
                      "复制",
                      style: TextStyle(
                          color: rgba(153, 153, 153, 1), fontSize: 11.rpx),
                    ),
                  )
                ],
              ),
            ),
          ),
          /*SizedBox(
            width: 20.rpx,
          ),
          InkWell(
              onTap: () async {
                String gps = await AmapUtils.getInstance().addressToGps(
                    model.getShopInfoEntity().shopInfo.address,
                    model.getCityName());
                MapUtils.getInstance().open(gps,model.getShopInfoEntity().shopInfo.address);
              },
              child: Container(
                child: Column(
                  children: [
                    Container(
                      height: 20.rpx,
                      width: 20.rpx,
                      child: LifeImage(
                        name: "assets/images/shop/ic_gps_black.png",
                        width: 20.rpx,
                        height: 20.rpx,
                      ),
                    ),
                    Container(
                      height: 18.rpx,
                      child: Text(
                        "导航",
                        style: TextStyle(
                            color: rgba(153, 153, 153, 1), fontSize: 11.rpx),
                      ),
                    )
                  ],
                ),
              ))*/
        ],
      ),
    );
  }
}
